@@ -24,8 +24,6 @@ class Mock(object):
     def __init__(self, *args, **kwargs):
         pass
 
-    __all__ = []
-
     def __call__(self, *args, **kwargs):
         ret = Mock()
         # If mocked function is used as a decorator, expose decorated function.
@@ -44,7 +42,6 @@ class Mock(object):
 MOCK_MODULES = [
     # salt core
     'Crypto',
-    'Crypto.Signature',
     'Crypto.Cipher',
     'Crypto.Hash',
     'Crypto.PublicKey',
@@ -97,13 +94,11 @@ MOCK_MODULES = [
     'MySQLdb',
     'MySQLdb.cursors',
     'psutil',
-    'psutil.version_info',
     'pycassa',
     'pymongo',
     'rabbitmq_server',
     'redis',
     'requests',
-    'requests.exceptions',
     'rpm',
     'rpmUtils',
     'rpmUtils.arch',
@@ -155,27 +150,8 @@ project = 'Salt'
 copyright = '2015 SaltStack, Inc.'
 
 version = salt.version.__version__
-latest_release = '2015.8.0'  # latest release
-previous_release = '2014.7.6'  # latest release from previous branch
-previous_release_dir = '2014.7'  # path on web server for previous branch
-build_type = 'inactive'  # latest, previous, develop, inactive
-
-# set release to 'version' for develop so sha is used
-# - otherwise -
-# set release to 'latest_release' or 'previous_release'
-
-release = previous_release # version, latest_release, previous_release
-
-# Set google custom search engine
-
-if release == latest_release:
-    search_cx = '004624818632696854117:yfmprrbw3pk' # latest
-elif release.startswith('2014.7'):
-    search_cx = '004624818632696854117:thhslradbru' # 2014.7
-elif release.startswith('2015.5'):
-    search_cx = '004624818632696854117:ovogwef29do' # 2015.5
-else:
-    search_cx = '004624818632696854117:haj7bjntf4s'  # develop
+#release = '.'.join(map(str, salt.version.__version_info__))
+release = '2014.7.2'
 
 needs_sphinx = '1.3'
 
@@ -214,20 +190,11 @@ autosummary_generate = True
 
 # Define a substitution for linking to the latest release tarball
 rst_prolog = """\
-.. |current_release_doc| replace:: :doc:`/topics/releases/{release}`
 .. |saltrepo| replace:: https://github.com/saltstack/salt
 .. _`salt-users`: https://groups.google.com/forum/#!forum/salt-users
 .. _`salt-announce`: https://groups.google.com/forum/#!forum/salt-announce
 .. _`salt-packagers`: https://groups.google.com/forum/#!forum/salt-packagers
-.. |windownload| raw:: html
-
-     <p>x86: <a href="https://repo.saltstack.com/windows/Salt-Minion-{release}-x86-Setup.exe"><strong>Salt-Minion-{release}-x86-Setup.exe</strong></a>
-      | <a href="https://repo.saltstack.com/windows/Salt-Minion-{release}-x86-Setup.exe.md5"><strong>md5</strong></a></p>
-
-      <p>AMD64: <a href="https://repo.saltstack.com/windows/Salt-Minion-{release}-AMD64-Setup.exe"><strong>Salt-Minion-{release}-AMD64-Setup.exe</strong></a>
-      | <a href="https://repo.saltstack.com/windows/Salt-Minion-{release}-AMD64-Setup.exe.md5"><strong>md5</strong></a></p>
-
-""".format(release=release)
+"""
 
 # A shortcut for linking to tickets on the GitHub issue tracker
 extlinks = {
@@ -245,7 +212,7 @@ gettext_compact = False
 
 
 ### HTML options
-html_theme = 'saltstack2' #change to 'saltstack' to use previous theme
+html_theme = 'saltstack'
 html_theme_path = ['_themes']
 html_title = u''
 html_short_title = 'Salt'
@@ -296,11 +263,6 @@ html_context = {
     'github_base': 'https://github.com/saltstack/salt',
     'github_issues': 'https://github.com/saltstack/salt/issues',
     'github_downloads': 'https://github.com/saltstack/salt/downloads',
-    'latest_release': latest_release,
-    'previous_release': previous_release,
-    'previous_release_dir': previous_release_dir,
-    'search_cx': search_cx,
-    'build_type': build_type,
 }
 
 html_use_index = True
@@ -312,24 +274,26 @@ html_show_copyright = True
 ### Latex options
 
 latex_documents = [
-  ('contents', 'Salt.tex', 'Salt Documentation', 'SaltStack, Inc.', 'manual'),
+  ('contents','Salt-All.tex','Salt All-In-One Documentation','SaltStack, Inc.','manual'),
+  ('contents-1','Salt-1.tex','Salt 1/4 Documentation','SaltStack, Inc.','manual'),
+  ('contents-2','Salt-2.tex','Salt 2/4 Documentation', 'SaltStack, Inc.','manual'),
+  ('contents-3','Salt-3.tex','Salt 3/4 Documentation','SaltStack, Inc.','manual'),
+  ('contents-4','Salt-4.tex','Salt 4/4 Documentation','SaltStack, Inc.','manual'),
 ]
 
-latex_logo = '_static/salt-logo.png'
+latex_logo = '_static/salt-logo.pdf'
 
 latex_elements = {
     'inputenc': '',     # use XeTeX instead of the inputenc LaTeX package.
     'utf8extra': '',
     'preamble': '''
-    \usepackage{fontspec}
-    \setsansfont{Linux Biolinum O}
-    \setromanfont{Linux Libertine O}
-    \setmonofont{Source Code Pro}
+
+\usepackage{fontspec}
+\setsansfont{DejaVu Sans}
+\setromanfont{DejaVu Serif}
+\setmonofont{DejaVu Sans Mono}
 ''',
 }
-### Linux Biolinum, Linux Libertine: http://www.linuxlibertine.org/
-### Source Code Pro: https://github.com/adobe-fonts/source-code-pro/releases
-
 
 ### Linkcheck options
 linkcheck_ignore = [r'http://127.0.0.1',
