@@ -13,6 +13,7 @@ import logging
 from salt.exceptions import CommandExecutionError, SaltInvocationError
 
 log = logging.getLogger(__name__)
+
 # Import third party libs
 from jinja2 import Environment
 try:
@@ -110,7 +111,7 @@ class _LDAPConnection(object):
             )
 
 
-def _bind(username, password, anonymous=False):
+def _bind(username, password):
     '''
     Authenticate via an LDAP bind
     '''
@@ -120,10 +121,8 @@ def _bind(username, password, anonymous=False):
     connargs = {}
     # config params (auth.ldap.*)
     params = {
-        'mandatory': ['uri', 'server', 'port', 'tls', 'no_verify', 'anonymous',
-                      'accountattributename', 'activedirectory'],
-        'additional': ['binddn', 'bindpw', 'filter', 'groupclass',
-                       'auth_by_group_membership_only'],
+        'mandatory': ['uri', 'server', 'port', 'tls', 'no_verify', 'anonymous', 'accountattributename', 'activedirectory'],
+        'additional': ['binddn', 'bindpw', 'filter', 'groupclass'],
     }
 
     paramvalues = {}
@@ -138,7 +137,6 @@ def _bind(username, password, anonymous=False):
         #except SaltInvocationError:
         #    pass
 
-    paramvalues['anonymous'] = anonymous
     if paramvalues['binddn']:
         # the binddn can also be composited, e.g.
         #   - {{ username }}@domain.com
@@ -206,10 +204,7 @@ def _bind(username, password, anonymous=False):
     # Update connection dictionary with the user's password
     connargs['bindpw'] = password
     # Attempt bind with user dn and password
-    if paramvalues['anonymous']:
-        log.debug('Attempting anonymous LDAP bind')
-    else:
-        log.debug('Attempting LDAP bind with user dn: {0}'.format(connargs['binddn']))
+    log.debug('Attempting LDAP bind with user dn: {0}'.format(connargs['binddn']))
     try:
         ldap_conn = _LDAPConnection(**connargs).ldap
     except Exception:
@@ -229,8 +224,8 @@ def auth(username, password):
     '''
     Simple LDAP auth
     '''
-    if _bind(username, password, anonymous=_config('auth_by_group_membership_only', mandatory=False) and
-                                           _config('anonymous', mandatory=False)):
+
+    if _bind(username, password):
         log.debug('LDAP authentication successful')
         return True
     else:
@@ -255,8 +250,7 @@ def groups(username, **kwargs):
     '''
     group_list = []
 
-    bind = _bind(username, kwargs['password'],
-                 anonymous=_config('anonymous', mandatory=False))
+    bind = _bind(username, kwargs['password'])
     if bind:
         log.debug('ldap bind to determine group membership succeeded!')
 
@@ -290,24 +284,15 @@ def groups(username, **kwargs):
                     group_list.append(entry['cn'][0])
             log.debug('User {0} is a member of groups: {1}'.format(username, group_list))
         else:
-            if _config('groupou'):
-                search_base = 'ou={0},{1}'.format(_config('groupou'), _config('basedn'))
-            else:
-                search_base = '{0}'.format(_config('basedn'))
-            search_string = '(&({0}={1})(objectClass={2}))'.format(_config('accountattributename'),
-                                                                   username, _config('groupclass'))
-            search_results = bind.search_s(search_base,
+            search_results = bind.search_s('ou={0},{1}'.format(_config('groupou'), _config('basedn')),
                                            ldap.SCOPE_SUBTREE,
-                                           search_string,
+                                           '(&({0}={1})(objectClass={2}))'.format(_config('accountattributename'),
+                                                                                  username, _config('groupclass')),
                                            [_config('accountattributename'), 'cn'])
             for _, entry in search_results:
                 if username in entry[_config('accountattributename')]:
                     group_list.append(entry['cn'][0])
             log.debug('User {0} is a member of groups: {1}'.format(username, group_list))
-
-            if not auth(username, kwargs['password']):
-                log.error('LDAP username and password do not match')
-                return []
     else:
         log.error('ldap bind to determine group membership FAILED!')
         return group_list
