@@ -5,6 +5,7 @@ Execute batch runs
 
 # Import python libs
 from __future__ import print_function
+from __future__ import absolute_import
 import math
 import time
 import copy
@@ -13,6 +14,7 @@ import copy
 import salt.client
 import salt.output
 from salt.utils import print_cli
+from salt.ext.six.moves import range
 
 
 class Batch(object):
@@ -42,13 +44,18 @@ class Batch(object):
         else:
             args.append(self.opts.get('expr_form', 'glob'))
 
-        ping_gen = self.local.cmd_iter(*args, **self.eauth)
+        ping_gen = self.local.cmd_iter_no_block(*args, **self.eauth)
+        wait_until = time.time() + self.opts['timeout']
 
         fret = set()
         for ret in ping_gen:
             m = next(ret.iterkeys())
             if m is not None:
                 fret.add(m)
+            if time.time() > wait_until:
+                break
+            if m is None:
+                time.sleep(0.1)
         return (list(fret), ping_gen)
 
     def get_bnum(self):
@@ -170,8 +177,7 @@ class Batch(object):
                                 parts[minion]['ret'] = {}
 
             for minion, data in parts.items():
-                if minion in active:
-                    active.remove(minion)
+                active.remove(minion)
                 if self.opts.get('raw'):
                     yield data
                 else:
