@@ -2,6 +2,7 @@
 '''
 Create ssh executor system
 '''
+from __future__ import absolute_import
 # Import python libs
 import os
 import tarfile
@@ -99,22 +100,20 @@ def lowstate_file_refs(chunks, extras=''):
     return refs
 
 
-def salt_refs(data, ret=None):
+def salt_refs(data):
     '''
     Pull salt file references out of the states
     '''
     proto = 'salt://'
-    if ret is None:
-        ret = []
+    ret = []
     if isinstance(data, str):
-        if data.startswith(proto) and data not in ret:
-            ret.append(data)
+        if data.startswith(proto):
+            return [data]
     if isinstance(data, list):
         for comp in data:
-            salt_refs(comp, ret)
-    if isinstance(data, dict):
-        for comp in data:
-            salt_refs(data[comp], ret)
+            if isinstance(comp, str):
+                if comp.startswith(proto):
+                    ret.append(comp)
     return ret
 
 
@@ -156,7 +155,7 @@ def prep_trans_tar(file_client, chunks, file_refs, pillar=None):
                     if not os.path.isdir(tgt_dir):
                         os.makedirs(tgt_dir)
                     shutil.copy(path, tgt)
-                    continue
+                    break
                 files = file_client.cache_dir(name, saltenv)
                 if files:
                     for filename in files:
@@ -172,7 +171,7 @@ def prep_trans_tar(file_client, chunks, file_refs, pillar=None):
                         if not os.path.isdir(tgt_dir):
                             os.makedirs(tgt_dir)
                         shutil.copy(filename, tgt)
-                    continue
+                    break
     cwd = os.getcwd()
     os.chdir(gendir)
     with closing(tarfile.open(trans_tar, 'w:gz')) as tfp:
