@@ -2,6 +2,8 @@
 '''
 Create ssh executor system
 '''
+
+from __future__ import absolute_import
 # Import python libs
 import os
 import copy
@@ -18,7 +20,7 @@ import salt.state
 import salt.loader
 import salt.minion
 import salt.log
-from salt._compat import string_types
+from salt.ext.six import string_types
 
 log = logging.getLogger(__name__)
 
@@ -109,15 +111,21 @@ def sls(mods, saltenv='base', test=None, exclude=None, env=None, **kwargs):
             trans_tar,
             '{0}/salt_state.tgz'.format(__opts__['thin_dir']))
     stdout, stderr, _ = single.cmd_block()
-    try:
-        return json.loads(stdout, object_hook=salt.utils.decode_dict)
-    except Exception, e:
-        log.error("JSON Render failed for: {0}\n{1}".format(stdout, stderr))
-        log.error(str(e))
+
+    # Clean up our tar
     try:
         os.remove(trans_tar)
     except (OSError, IOError):
         pass
+
+    # Read in the JSON data and return the data structure
+    try:
+        return json.loads(stdout, object_hook=salt.utils.decode_dict)
+    except Exception as e:
+        log.error("JSON Render failed for: {0}\n{1}".format(stdout, stderr))
+        log.error(str(e))
+
+    # If for some reason the json load fails, return the stdout
     return stdout
 
 
@@ -169,11 +177,22 @@ def low(data, **kwargs):
             trans_tar,
             '{0}/salt_state.tgz'.format(__opts__['thin_dir']))
     stdout, stderr, _ = single.cmd_block()
+
+    # Clean up our tar
     try:
         os.remove(trans_tar)
     except (OSError, IOError):
         pass
-    return json.loads(stdout, object_hook=salt.utils.decode_dict)
+
+    # Read in the JSON data and return the data structure
+    try:
+        return json.loads(stdout, object_hook=salt.utils.decode_dict)
+    except Exception as e:
+        log.error("JSON Render failed for: {0}\n{1}".format(stdout, stderr))
+        log.error(str(e))
+
+    # If for some reason the json load fails, return the stdout
+    return stdout
 
 
 def high(data, **kwargs):
@@ -187,6 +206,7 @@ def high(data, **kwargs):
 
         salt '*' state.high '{"vim": {"pkg": ["installed"]}}'
     '''
+    __pillar__.update(kwargs.get('pillar', {}))
     st_kwargs = __salt__.kwargs
     __opts__['grains'] = __grains__
     st_ = salt.client.ssh.state.SSHHighState(
@@ -221,11 +241,22 @@ def high(data, **kwargs):
             trans_tar,
             '{0}/salt_state.tgz'.format(__opts__['thin_dir']))
     stdout, stderr, _ = single.cmd_block()
+
+    # Clean up our tar
     try:
         os.remove(trans_tar)
     except (OSError, IOError):
         pass
-    return json.loads(stdout, object_hook=salt.utils.decode_dict)
+
+    # Read in the JSON data and return the data structure
+    try:
+        return json.loads(stdout, object_hook=salt.utils.decode_dict)
+    except Exception as e:
+        log.error("JSON Render failed for: {0}\n{1}".format(stdout, stderr))
+        log.error(str(e))
+
+    # If for some reason the json load fails, return the stdout
+    return stdout
 
 
 def highstate(test=None, **kwargs):
@@ -241,6 +272,7 @@ def highstate(test=None, **kwargs):
         salt '*' state.highstate exclude=sls_to_exclude
         salt '*' state.highstate exclude="[{'id': 'id_to_exclude'}, {'sls': 'sls_to_exclude'}]"
     '''
+    __pillar__.update(kwargs.get('pillar', {}))
     st_kwargs = __salt__.kwargs
     __opts__['grains'] = __grains__
     st_ = salt.client.ssh.state.SSHHighState(
@@ -280,15 +312,21 @@ def highstate(test=None, **kwargs):
             trans_tar,
             '{0}/salt_state.tgz'.format(__opts__['thin_dir']))
     stdout, stderr, _ = single.cmd_block()
-    try:
-        stdout = json.loads(stdout, object_hook=salt.utils.decode_dict)
-    except Exception, e:
-        log.error('JSON Render failed for: {0}\n{1}'.format(stdout, stderr))
-        log.error(str(e))
+
+    # Clean up our tar
     try:
         os.remove(trans_tar)
     except (OSError, IOError):
         pass
+
+    # Read in the JSON data and return the data structure
+    try:
+        return json.loads(stdout, object_hook=salt.utils.decode_dict)
+    except Exception as e:
+        log.error("JSON Render failed for: {0}\n{1}".format(stdout, stderr))
+        log.error(str(e))
+
+    # If for some reason the json load fails, return the stdout
     return stdout
 
 
@@ -304,6 +342,7 @@ def top(topfn, test=None, **kwargs):
         salt '*' state.top reverse_top.sls exclude=sls_to_exclude
         salt '*' state.top reverse_top.sls exclude="[{'id': 'id_to_exclude'}, {'sls': 'sls_to_exclude'}]"
     '''
+    __pillar__.update(kwargs.get('pillar', {}))
     st_kwargs = __salt__.kwargs
     __opts__['grains'] = __grains__
     if salt.utils.test_mode(test=test, **kwargs):
@@ -344,11 +383,22 @@ def top(topfn, test=None, **kwargs):
             trans_tar,
             '{0}/salt_state.tgz'.format(__opts__['thin_dir']))
     stdout, stderr, _ = single.cmd_block()
+
+    # Clean up our tar
     try:
         os.remove(trans_tar)
     except (OSError, IOError):
         pass
-    return json.loads(stdout, object_hook=salt.utils.decode_dict)
+
+    # Read in the JSON data and return the data structure
+    try:
+        return json.loads(stdout, object_hook=salt.utils.decode_dict)
+    except Exception as e:
+        log.error("JSON Render failed for: {0}\n{1}".format(stdout, stderr))
+        log.error(str(e))
+
+    # If for some reason the json load fails, return the stdout
+    return stdout
 
 
 def show_highstate():
@@ -400,6 +450,7 @@ def show_sls(mods, saltenv='base', test=None, env=None, **kwargs):
 
         salt '*' state.show_sls core,edit.vim dev
     '''
+    __pillar__.update(kwargs.get('pillar', {}))
     __opts__['grains'] = __grains__
     if env is not None:
         salt.utils.warn_until(
@@ -460,3 +511,119 @@ def show_top():
         return errors
     matches = st_.top_matches(top_data)
     return matches
+
+
+def single(fun, name, test=None, **kwargs):
+    '''
+    .. versionadded:: 2015.5.0
+
+    Execute a single state function with the named kwargs, returns False if
+    insufficient data is sent to the command
+
+    By default, the values of the kwargs will be parsed as YAML. So, you can
+    specify lists values, or lists of single entry key-value maps, as you
+    would in a YAML salt file. Alternatively, JSON format of keyword values
+    is also supported.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' state.single pkg.installed name=vim
+
+    '''
+    st_kwargs = __salt__.kwargs
+    __opts__['grains'] = __grains__
+
+    # state.fun -> [state, fun]
+    comps = fun.split('.')
+    if len(comps) < 2:
+        __context__['retcode'] = 1
+        return 'Invalid function passed'
+
+    # Create the low chunk, using kwargs as a base
+    kwargs.update({'state': comps[0],
+                   'fun': comps[1],
+                   '__id__': name,
+                   'name': name})
+
+    opts = copy.deepcopy(__opts__)
+
+    # Set test mode
+    if salt.utils.test_mode(test=test, **kwargs):
+        opts['test'] = True
+    else:
+        opts['test'] = __opts__.get('test', None)
+
+    # Get the override pillar data
+    __pillar__.update(kwargs.get('pillar', {}))
+
+    # Create the State environment
+    st_ = salt.client.ssh.state.SSHState(__opts__, __pillar__)
+
+    # Verify the low chunk
+    err = st_.verify_data(kwargs)
+    if err:
+        __context__['retcode'] = 1
+        return err
+
+    # Must be a list of low-chunks
+    chunks = [kwargs]
+
+    # Retrieve file refs for the state run, so we can copy relevant files down
+    # to the minion before executing the state
+    file_refs = salt.client.ssh.state.lowstate_file_refs(
+            chunks,
+            _merge_extra_filerefs(
+                kwargs.get('extra_filerefs', ''),
+                __opts__.get('extra_filerefs', '')
+                )
+            )
+
+    # Create the tar containing the state pkg and relevant files
+    trans_tar = salt.client.ssh.state.prep_trans_tar(
+            __context__['fileclient'],
+            chunks,
+            file_refs,
+            __pillar__)
+
+    # Create a hash so we can verify the tar on the target system
+    trans_tar_sum = salt.utils.get_hash(trans_tar, __opts__['hash_type'])
+
+    # We use state.pkg to execute the "state package"
+    cmd = 'state.pkg {0}/salt_state.tgz test={1} pkg_sum={2} hash_type={3}'.format(
+            __opts__['thin_dir'],
+            test,
+            trans_tar_sum,
+            __opts__['hash_type'])
+
+    # Create a salt-ssh Single object to actually do the ssh work
+    single = salt.client.ssh.Single(
+            __opts__,
+            cmd,
+            fsclient=__context__['fileclient'],
+            **st_kwargs)
+
+    # Copy the tar down
+    single.shell.send(
+            trans_tar,
+            '{0}/salt_state.tgz'.format(__opts__['thin_dir']))
+
+    # Run the state.pkg command on the target
+    stdout, stderr, _ = single.cmd_block()
+
+    # Clean up our tar
+    try:
+        os.remove(trans_tar)
+    except (OSError, IOError):
+        pass
+
+    # Read in the JSON data and return the data structure
+    try:
+        return json.loads(stdout, object_hook=salt.utils.decode_dict)
+    except Exception as e:
+        log.error("JSON Render failed for: {0}\n{1}".format(stdout, stderr))
+        log.error(str(e))
+
+    # If for some reason the json load fails, return the stdout
+    return stdout
