@@ -4,6 +4,8 @@ The top level interface used to translate configuration data back to the
 correct cloud modules
 '''
 
+from __future__ import absolute_import
+
 # Import python libs
 from __future__ import print_function, generators
 import copy
@@ -15,6 +17,7 @@ import signal
 import logging
 import multiprocessing
 from itertools import groupby
+from salt.ext.six.moves import input
 
 # Import salt.cloud libs
 from salt.exceptions import (
@@ -32,7 +35,8 @@ import salt.utils
 import salt.utils.cloud
 import salt.syspaths
 from salt.utils import context
-from salt._compat import string_types
+from salt.ext.six import string_types
+from salt.template import compile_template
 
 # Import third party libs
 import yaml
@@ -40,14 +44,6 @@ import yaml
 # Get logging started
 log = logging.getLogger(__name__)
 
-try:
-    from mako.template import Template
-    from mako.exceptions import MakoException
-    MAKO_AVAILABLE = True
-except ImportError:
-    log.debug('Mako not available')
-    MAKO_AVAILABLE = False
-
 
 def communicator(func):
     '''Warning, this is a picklable decorator !'''
@@ -58,12 +54,12 @@ def communicator(func):
         try:
             ret = func(*args, **kw)
             queue.put('END')
-        except KeyboardInterrupt, ex:
+        except KeyboardInterrupt as ex:
             trace = traceback.format_exc()
             queue.put('KEYBOARDINT')
             queue.put('Keyboard interrupt')
             queue.put('{0}\n{1}\n'.format(ex, trace))
-        except Exception, ex:
+        except Exception as ex:
             trace = traceback.format_exc()
             queue.put('ERROR')
             queue.put('Exception')
@@ -82,7 +78,7 @@ def enter_mainloop(target,
                    queue=None):
     '''Manage a multiprocessing pool
 
-    - If the queue does not output anything, the pool runs indefinitvly
+    - If the queue does not output anything, the pool runs indefinitely
 
     - If the queue returns KEYBOARDINT or ERROR, this will kill the pool
       totally calling terminate & join and ands with a SaltCloudSystemExit
@@ -100,14 +96,14 @@ def enter_mainloop(target,
         pool size if you did not provide yourself a pool
     callback
         a boolean taking a string in argument which returns True to
-        signal that 'target' is finnished and we need to join
+        signal that 'target' is finished and we need to join
         the pool
     queue
         A custom multiproccessing queue in case you want to do
         extra stuff and need it later in your program
     args
-        positionnal arguments to call the function with
-        if you dont want to use pool.map
+        positional arguments to call the function with
+        if you don't want to use pool.map
 
     mapped_args
         a list of one or more arguments combinations to call the function with
@@ -185,7 +181,7 @@ class CloudClient(object):
                 self.opts['providers'].update({name: {driver: provider}})
             for name, profile in pillars.pop('profiles', {}).items():
                 provider = profile['provider'].split(':')[0]
-                driver = self.opts['providers'][provider].iterkeys().next()
+                driver = next(iter(self.opts['providers'][provider].keys()))
                 profile['provider'] = '{0}:{1}'.format(provider, driver)
                 profile['profile'] = name
                 self.opts['profiles'].update({name: profile})
@@ -218,11 +214,11 @@ class CloudClient(object):
             for _profile in [a for a in opts.get('profiles', {})]:
                 if not _profile == profile:
                     opts['profiles'].pop(_profile)
-            # if profile is specified and we have enougth info about providers
+            # if profile is specified and we have enough info about providers
             # also filter them to speedup methods like
             # __filter_non_working_providers
             providers = [a.get('provider', '').split(':')[0]
-                         for a in opts['profiles'].itervalues()
+                         for a in opts['profiles'].values()
                          if a.get('provider', '')]
             if providers:
                 _providers = opts.get('providers', {})
@@ -355,7 +351,7 @@ class CloudClient(object):
         '''
         Destroy the named VMs
         '''
-        mapper = salt.cloud.Map(self._opts_defaults(destroy=True))
+        mapper = salt.cloud.Map(self._opts_defaults())
         if isinstance(names, str):
             names = names.split(',')
         return salt.utils.cloud.simple_types_filter(
@@ -371,14 +367,14 @@ class CloudClient(object):
         .. code-block:: python
 
             client.create(names=['myinstance'], provider='my-ec2-config',
-                kwargs={'image': 'ami-1624987f', 'size': 'Micro Instance',
+                kwargs={'image': 'ami-1624987f', 'size': 't1.micro',
                         'ssh_username': 'ec2-user', 'securitygroup': 'default',
                         'delvol_on_destroy': True})
         '''
         mapper = salt.cloud.Map(self._opts_defaults())
         providers = self.opts['providers']
         if provider in providers:
-            provider += ':{0}'.format(providers[provider].iterkeys().next())
+            provider += ':{0}'.format(next(iter(providers[provider].keys())))
         else:
             return False
         if isinstance(names, str):
@@ -411,7 +407,7 @@ class CloudClient(object):
         mapper = salt.cloud.Map(self._opts_defaults())
         providers = mapper.map_providers_parallel()
         if provider in providers:
-            provider += ':{0}'.format(providers[provider].iterkeys().next())
+            provider += ':{0}'.format(next(iter(providers[provider].keys())))
         else:
             return False
         if isinstance(names, str):
@@ -450,7 +446,7 @@ class CloudClient(object):
                 kwargs={'image': 'ami-10314d79'}
             )
         '''
-        mapper = salt.cloud.Map(self._opts_defaults(action=fun, names=names))
+        mapper = salt.cloud.Map(self._opts_defaults(action=fun))
         if names and not provider:
             self.opts['action'] = fun
             return mapper.do_action(names, kwargs)
@@ -487,7 +483,7 @@ class Cloud(object):
         Return the configured providers
         '''
         providers = set()
-        for alias, drivers in self.opts['providers'].iteritems():
+        for alias, drivers in self.opts['providers'].items():
             if len(drivers) > 1:
                 for driver in drivers:
                     providers.add('{0}:{1}'.format(alias, driver))
@@ -503,7 +499,7 @@ class Cloud(object):
             lookup = 'all'
         if lookup == 'all':
             providers = set()
-            for alias, drivers in self.opts['providers'].iteritems():
+            for alias, drivers in self.opts['providers'].items():
                 for driver in drivers:
                     providers.add((alias, driver))
 
@@ -527,7 +523,7 @@ class Cloud(object):
             return set((alias, driver))
 
         providers = set()
-        for alias, drivers in self.opts['providers'].iteritems():
+        for alias, drivers in self.opts['providers'].items():
             for driver in drivers:
                 if lookup in (alias, driver):
                     providers.add((alias, driver))
@@ -541,6 +537,40 @@ class Cloud(object):
             )
         return providers
 
+    def lookup_profiles(self, provider, lookup):
+        '''
+        Return a dictionary describing the configured profiles
+        '''
+        if provider is None:
+            provider = 'all'
+        if lookup is None:
+            lookup = 'all'
+
+        if lookup == 'all':
+            profiles = set()
+            provider_profiles = set()
+            for alias, info in self.opts['profiles'].items():
+                providers = info.get('provider')
+
+                if providers:
+                    given_prov_name = providers.split(':')[0]
+                    salt_prov_name = providers.split(':')[1]
+                    if given_prov_name == provider:
+                        provider_profiles.add((alias, given_prov_name))
+                    elif salt_prov_name == provider:
+                        provider_profiles.add((alias, salt_prov_name))
+                    profiles.add((alias, given_prov_name))
+
+            if not profiles:
+                raise SaltCloudSystemExit(
+                    'There are no cloud profiles configured.'
+                )
+
+            if provider != 'all':
+                return provider_profiles
+
+            return profiles
+
     def map_providers(self, query='list_nodes', cached=False):
         '''
         Return a mapping of what named VMs are running on what VM providers
@@ -550,8 +580,8 @@ class Cloud(object):
             return self.__cached_provider_queries[query]
 
         pmap = {}
-        for alias, drivers in self.opts['providers'].iteritems():
-            for driver, details in drivers.iteritems():
+        for alias, drivers in self.opts['providers'].items():
+            for driver, details in drivers.items():
                 fun = '{0}.{1}'.format(driver, query)
                 if fun not in self.clouds:
                     log.error(
@@ -598,10 +628,10 @@ class Cloud(object):
 
         # Optimize Providers
         opts['providers'] = self._optimize_providers(opts['providers'])
-        for alias, drivers in opts['providers'].iteritems():
+        for alias, drivers in opts['providers'].items():
             # Make temp query for this driver to avoid overwrite next
             this_query = query
-            for driver, details in drivers.iteritems():
+            for driver, details in drivers.items():
                 # If driver has function list_nodes_min, just replace it
                 # with query param to check existing vms on this driver
                 # for minimum information, Otherwise still use query param.
@@ -653,18 +683,18 @@ class Cloud(object):
         matches = {}
         handled_drivers = {}
         mapped_providers = self.map_providers_parallel(query, cached=cached)
-        for alias, drivers in mapped_providers.iteritems():
-            for driver, vms in drivers.iteritems():
+        for alias, drivers in mapped_providers.items():
+            for driver, vms in drivers.items():
                 if driver not in handled_drivers:
                     handled_drivers[driver] = alias
                 # When a profile is specified, only return an instance
                 # that matches the provider specified in the profile.
                 # This solves the issues when many providers return the
                 # same instance. For example there may be one provider for
-                # each avaliablity zone in amazon in the same region, but
+                # each availability zone in amazon in the same region, but
                 # the search returns the same instance for each provider
                 # because amazon returns all instances in a region, not
-                # avaliabilty zone.
+                # availability zone.
                 if profile:
                     if alias not in \
                         self.opts['profiles'][profile]['provider'].split(
@@ -672,7 +702,7 @@ class Cloud(object):
                             )[0]:
                         continue
 
-                for vm_name, details in vms.iteritems():
+                for vm_name, details in vms.items():
                     # XXX: The logic bellow can be removed once the aws driver
                     # is removed
                     if vm_name not in names:
@@ -702,14 +732,14 @@ class Cloud(object):
         new_providers = {}
         provider_by_driver = {}
 
-        for alias, driver in providers.iteritems():
-            for name, data in driver.iteritems():
+        for alias, driver in providers.items():
+            for name, data in driver.items():
                 if name not in provider_by_driver:
                     provider_by_driver[name] = {}
 
                 provider_by_driver[name][alias] = data
 
-        for driver, providers_data in provider_by_driver.iteritems():
+        for driver, providers_data in provider_by_driver.items():
             fun = '{0}.optimize_providers'.format(driver)
             if fun not in self.clouds:
                 log.debug(
@@ -718,7 +748,7 @@ class Cloud(object):
                     )
                 )
 
-                for name, prov_data in providers_data.iteritems():
+                for name, prov_data in providers_data.items():
                     if name not in new_providers:
                         new_providers[name] = {}
                     new_providers[name][driver] = prov_data
@@ -726,7 +756,7 @@ class Cloud(object):
 
             new_data = self.clouds[fun](providers_data)
             if new_data:
-                for name, prov_data in new_data.iteritems():
+                for name, prov_data in new_data.items():
                     if name not in new_providers:
                         new_providers[name] = {}
                     new_providers[name][driver] = prov_data
@@ -878,13 +908,30 @@ class Cloud(object):
                 data[alias][driver] = {}
         return data
 
+    def profile_list(self, provider, lookup='all'):
+        '''
+        Return a mapping of all configured profiles
+        '''
+        data = {}
+        lookups = self.lookup_profiles(provider, lookup)
+
+        if not lookups:
+            return data
+
+        for alias, driver in lookups:
+            if alias not in data:
+                data[alias] = {}
+            if driver not in data[alias]:
+                data[alias][driver] = {}
+        return data
+
     def create_all(self):
         '''
         Create/Verify the VMs in the VM data
         '''
         ret = []
 
-        for vm_name, vm_details in self.opts['profiles'].iteritems():
+        for vm_name, vm_details in self.opts['profiles'].items():
             ret.append(
                 {vm_name: self.create(vm_details)}
             )
@@ -900,8 +947,8 @@ class Cloud(object):
         matching = self.get_running_by_names(names, cached=cached)
         vms_to_destroy = set()
         parallel_data = []
-        for alias, drivers in matching.iteritems():
-            for driver, vms in drivers.iteritems():
+        for alias, drivers in matching.items():
+            for driver, vms in drivers.items():
                 for name in vms:
                     if name in names:
                         vms_to_destroy.add((alias, driver, name))
@@ -1028,7 +1075,7 @@ class Cloud(object):
                     print(' {0}: {1}'.format(
                         idx, os.path.basename(filename)
                     ))
-                selection = raw_input(
+                selection = input(
                     'Which minion key should be deleted(number)? '
                 )
                 try:
@@ -1045,7 +1092,7 @@ class Cloud(object):
                 except Exception:
                     continue
 
-                delete = raw_input(
+                delete = input(
                     'Delete {0!r}? [Y/n]? '.format(filename)
                 )
                 if delete == '' or delete.lower().startswith('y'):
@@ -1285,14 +1332,13 @@ class Cloud(object):
         try:
             with salt.utils.fopen(self.opts['conf_file'], 'r') as mcc:
                 main_cloud_config = yaml.safe_load(mcc)
+            if not main_cloud_config:
+                main_cloud_config = {}
         except KeyError:
             main_cloud_config = {}
         except IOError:
             main_cloud_config = {}
 
-        if main_cloud_config is None:
-            main_cloud_config = {}
-
         profile_details = self.opts['profiles'][profile]
         alias, driver = profile_details['provider'].split(':')
         mapped_providers = self.map_providers_parallel()
@@ -1361,10 +1407,10 @@ class Cloud(object):
         ret = {}
         names = set(names)
 
-        for alias, drivers in self.map_providers_parallel().iteritems():
+        for alias, drivers in self.map_providers_parallel().items():
             if not names:
                 break
-            for driver, vms in drivers.iteritems():
+            for driver, vms in drivers.items():
                 if not names:
                     break
                 fun = '{0}.{1}'.format(driver, self.opts['action'])
@@ -1375,7 +1421,7 @@ class Cloud(object):
                         )
                     )
                     continue
-                for vm_name, vm_details in vms.iteritems():
+                for vm_name, vm_details in vms.items():
                     if not names:
                         break
                     if vm_name not in names:
@@ -1461,7 +1507,7 @@ class Cloud(object):
         '''
         Remove any mis-configured cloud providers from the available listing
         '''
-        for alias, drivers in self.opts['providers'].copy().iteritems():
+        for alias, drivers in self.opts['providers'].copy().items():
             for driver in drivers.copy():
                 fun = '{0}.get_configured_provider'.format(driver)
                 if fun not in self.clouds:
@@ -1531,9 +1577,9 @@ class Map(Cloud):
                 continue
 
             matching = self.get_running_by_names(names, query, cached)
-            for alias, drivers in matching.iteritems():
-                for driver, vms in drivers.iteritems():
-                    for vm_name, vm_details in vms.iteritems():
+            for alias, drivers in matching.items():
+                for driver, vms in drivers.items():
+                    for vm_name, vm_details in vms.items():
                         if alias not in interpolated_map:
                             interpolated_map[alias] = {}
                         if driver not in interpolated_map[alias]:
@@ -1557,14 +1603,11 @@ class Map(Cloud):
 
     def delete_map(self, query=None):
         query_map = self.interpolated_map(query=query)
-        for alias, drivers in query_map.copy().iteritems():
-            for driver, vms in drivers.copy().iteritems():
-                for vm_name, vm_details in vms.copy().iteritems():
+        for alias, drivers in query_map.copy().items():
+            for driver, vms in drivers.copy().items():
+                for vm_name, vm_details in vms.copy().items():
                     if vm_details == 'Absent':
                         query_map[alias][driver].pop(vm_name)
-                    elif vm_details['state'].lower() not in ('running',
-                                                             'active'):
-                        query_map[alias][driver].pop(vm_name)
                 if not query_map[alias][driver]:
                     query_map[alias].pop(driver)
             if not query_map[alias]:
@@ -1579,9 +1622,9 @@ class Map(Cloud):
             'reboot': ['running', 'active'],
         }
         vm_names = []
-        for alias, drivers in query_map.iteritems():
-            for driver, vms in drivers.iteritems():
-                for vm_name, vm_details in vms.iteritems():
+        for alias, drivers in query_map.items():
+            for driver, vms in drivers.items():
+                for vm_name, vm_details in vms.items():
                     if (vm_details != 'Absent') and \
                         (
                             vm_details['state'].lower() in
@@ -1590,17 +1633,6 @@ class Map(Cloud):
                         vm_names.append(vm_name)
         return vm_names
 
-    def _mako_read(self, fp):
-        try:
-            # open mako file
-            temp_ = Template(fp.read())
-            # render as yaml
-            yaml_str_ = temp_.render()
-            map_ = yaml.safe_load(yaml_str_)
-        except MakoException:
-            map_ = yaml.safe_load(fp.read())
-        return map_
-
     def read(self):
         '''
         Read in the specified map file and return the map structure
@@ -1615,13 +1647,11 @@ class Map(Cloud):
                 )
             )
         try:
-            with salt.utils.fopen(self.opts['map'], 'rb') as fp_:
-                if MAKO_AVAILABLE:
-                    log.warn('DEPRECATED: Mako will no longer be the default '
-                        'renderer for Salt Cloud maps in the Lithium release')
-                    map_ = self._mako_read(fp_)
-                else:
-                    map_ = yaml.safe_load(fp_.read())
+            renderer = self.opts.get('renderer', 'yaml_jinja')
+            rend = salt.loader.render(self.opts, {})
+            map_ = compile_template(
+                self.opts['map'], rend, renderer
+            )
         except Exception as exc:
             log.error(
                 'Rendering map {0} failed, render error:\n{1}'.format(
@@ -1646,7 +1676,7 @@ class Map(Cloud):
                         #   - bar1
                         #   - bar2
                         mapping = {mapping: None}
-                    for name, overrides in mapping.iteritems():
+                    for name, overrides in mapping.items():
                         if overrides is None:
                             # Foo:
                             #   - bar1:
@@ -1667,7 +1697,7 @@ class Map(Cloud):
                 #    grains:
                 #      foo: bar
                 entries = {}
-                for name, overrides in mapped.iteritems():
+                for name, overrides in mapped.items():
                     overrides.setdefault('name', name)
                     entries[name] = overrides
                 map_[profile] = entries
@@ -1685,7 +1715,7 @@ class Map(Cloud):
 
     def _has_loop(self, dmap, seen=None, val=None):
         if seen is None:
-            for values in dmap['create'].itervalues():
+            for values in dmap['create'].values():
                 seen = []
                 try:
                     machines = values['requires']
@@ -1737,7 +1767,7 @@ class Map(Cloud):
         pmap = self.map_providers_parallel(cached=cached)
         exist = set()
         defined = set()
-        for profile_name, nodes in self.rendered_map.iteritems():
+        for profile_name, nodes in self.rendered_map.items():
             if profile_name not in self.opts['profiles']:
                 msg = (
                     'The required profile, {0!r}, defined in the map '
@@ -1754,7 +1784,7 @@ class Map(Cloud):
                 continue
 
             profile_data = self.opts['profiles'].get(profile_name)
-            for nodename, overrides in nodes.iteritems():
+            for nodename, overrides in nodes.items():
                 # Get the VM name
                 nodedata = copy.deepcopy(profile_data)
                 # Update profile data with the map overrides
@@ -1794,17 +1824,17 @@ class Map(Cloud):
 
         def get_matching_by_name(name):
             matches = {}
-            for alias, drivers in pmap.iteritems():
-                for driver, vms in drivers.iteritems():
-                    for vm_name, details in vms.iteritems():
+            for alias, drivers in pmap.items():
+                for driver, vms in drivers.items():
+                    for vm_name, details in vms.items():
                         if vm_name == name:
                             if driver not in matches:
                                 matches[driver] = details['state']
             return matches
 
-        for alias, drivers in pmap.iteritems():
-            for driver, vms in drivers.iteritems():
-                for name, details in vms.iteritems():
+        for alias, drivers in pmap.items():
+            for driver, vms in drivers.items():
+                for name, details in vms.items():
                     exist.add((alias, driver, name))
                     if name not in ret['create']:
                         continue
@@ -1815,7 +1845,7 @@ class Map(Cloud):
                         continue
 
                     # A machine by the same name exists
-                    for mdriver, state in matching.iteritems():
+                    for mdriver, state in matching.items():
                         if name not in ret['create']:
                             # Machine already removed
                             break
@@ -1894,10 +1924,10 @@ class Map(Cloud):
         master_host = None
         master_finger = None
         try:
-            master_name, master_profile = (
+            master_name, master_profile = next((
                 (name, profile) for name, profile in create_list
                 if profile.get('make_master', False) is True
-            ).next()
+            ))
             master_minion_name = master_name
             log.debug('Creating new master {0!r}'.format(master_name))
             if salt.config.get_cloud_config_value(
@@ -1998,7 +2028,7 @@ class Map(Cloud):
                 out.pop('deploy_kwargs', {})
             )
 
-            master_host = deploy_kwargs.get('host', None)
+            master_host = deploy_kwargs.get('salt_host', deploy_kwargs.get('host', None))
             if master_host is None:
                 raise SaltCloudSystemExit(
                     'Host for new master {0} was not found, '
@@ -2088,7 +2118,7 @@ class Map(Cloud):
             if self.opts['start_action']:
                 actionlist = []
                 grp = -1
-                for key, val in groupby(dmap['create'].itervalues(),
+                for key, val in groupby(iter(dmap['create'].values()),
                                         lambda x: x['level']):
                     actionlist.append([])
                     grp += 1
@@ -2108,7 +2138,7 @@ class Map(Cloud):
                         timeout=self.opts['timeout'] * 60, expr_form='list'
                     ))
                 for obj in output_multip:
-                    obj.itervalues().next()['ret'] = out[obj.iterkeys().next()]
+                    next(iter(obj.values()))['ret'] = out[next(iter(obj.keys()))]
                     output.update(obj)
             else:
                 for obj in output_multip:
