@@ -45,6 +45,7 @@ Walkthrough <tutorial-gitfs>`.
 .. _libgit2: https://libgit2.github.com/
 .. _dulwich: https://www.samba.org/~jelmer/dulwich/
 '''
+from __future__ import absolute_import
 
 # Import python libs
 import copy
@@ -60,7 +61,7 @@ import shutil
 import stat
 import subprocess
 from datetime import datetime
-from salt._compat import text_type as _text_type
+from salt.ext.six import text_type as _text_type
 from salt._compat import StringIO
 
 VALID_PROVIDERS = ('gitpython', 'pygit2', 'dulwich')
@@ -99,7 +100,7 @@ _INVALID_REPO = (
 # Import salt libs
 import salt.utils
 import salt.fileserver
-from salt._compat import string_types
+from salt.ext.six import string_types
 from salt.exceptions import FileserverConfigError
 from salt.utils.event import tagify
 
@@ -388,12 +389,13 @@ def _get_tree_gitpython(repo, tgt_env):
     # Branch or tag not matched, check if 'tgt_env' is a commit
     if not _env_is_exposed(tgt_env):
         return None
-
     try:
         commit = repo['repo'].rev_parse(tgt_env)
+    except gitdb.exc.BadObject:
+        pass
+    else:
         return commit.tree
-    except gitdb.exc.ODBError:
-        return None
+    return None
 
 
 def _get_tree_pygit2(repo, tgt_env):
