@@ -33,6 +33,7 @@ Connection module for Amazon Route53
 
 :depends: boto
 '''
+from __future__ import absolute_import
 
 # Import Python libs
 import logging
@@ -49,7 +50,7 @@ try:
 except ImportError:
     HAS_BOTO = False
 
-from salt._compat import string_types
+from salt.ext.six import string_types
 import salt.utils.odict as odict
 
 
