@@ -13,13 +13,16 @@ Module to provide MongoDB functionality to Salt
     This data can also be passed into pillar. Options passed into opts will
     overwrite options passed into pillar.
 '''
+from __future__ import absolute_import
 
 # Import python libs
 import logging
 from distutils.version import LooseVersion  # pylint: disable=import-error,no-name-in-module
+import json
 
 # Import salt libs
-from salt._compat import string_types
+from salt.ext.six import string_types
+
 
 # Import third party libs
 try:
@@ -255,3 +258,103 @@ def user_remove(name, user=None, password=None, host=None, port=None,
         return str(err)
 
     return True
+
+
+def _to_dict(objects):
+    """
+    Potentially interprets a string as JSON for usage with mongo
+    """
+    try:
+        if isinstance(objects, string_types):
+            objects = json.loads(objects)
+    except ValueError as err:
+        log.error("Could not parse objects: %s", err)
+        raise err
+
+    return objects
+
+
+def insert(objects, collection, user=None, password=None,
+           host=None, port=None, database='admin'):
+    """
+    Insert an object or list of objects into a collection
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' mongodb.insert '[{"foo": "FOO", "bar": "BAR"}, {"foo": "BAZ", "bar": "BAM"}]' mycollection <user> <password> <host> <port> <database>
+
+    """
+    conn = _connect(user, password, host, port, database)
+    if not conn:
+        return "Failed to connect to mongo database"
+
+    try:
+        objects = _to_dict(objects)
+    except Exception, err:
+        return err
+
+    try:
+        log.info("Inserting %r into %s.%s", objects, database, collection)
+        mdb = pymongo.database.Database(conn, database)
+        col = getattr(mdb, collection)
+        ids = col.insert(objects)
+        return ids
+    except pymongo.errors.PyMongoError as err:
+        log.error("Inserting objects %r failed with error %s", objects, err)
+        return err
+
+
+def find(collection, query=None, user=None, password=None,
+         host=None, port=None, database='admin'):
+    conn = _connect(user, password, host, port)
+    if not conn:
+        return 'Failed to connect to mongo database'
+
+    try:
+        query = _to_dict(query)
+    except Exception, err:
+        return err
+
+    try:
+        log.info("Searching for %r in %s", query, collection)
+        mdb = pymongo.database.Database(conn, database)
+        col = getattr(mdb, collection)
+        ret = col.find(query)
+        return list(ret)
+    except pymongo.errors.PyMongoError as err:
+        log.error("Removing objects failed with error: %s", err)
+        return err
+
+
+def remove(collection, query=None, user=None, password=None,
+           host=None, port=None, database='admin', w=1):
+    """
+    Remove an object or list of objects into a collection
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' mongodb.remove mycollection '[{"foo": "FOO", "bar": "BAR"}, {"foo": "BAZ", "bar": "BAM"}]' <user> <password> <host> <port> <database>
+
+    """
+    conn = _connect(user, password, host, port)
+    if not conn:
+        return 'Failed to connect to mongo database'
+
+    try:
+        query = _to_dict(query)
+    except Exception, err:
+        return err.message
+
+    try:
+        log.info("Removing %r from %s", query, collection)
+        mdb = pymongo.database.Database(conn, database)
+        col = getattr(mdb, collection)
+        ret = col.remove(query, w=w)
+        return "{0} objects removed".format(ret['n'])
+    except pymongo.errors.PyMongoError as err:
+        log.error("Removing objects failed with error: %s", err.message)
+        return err.message
