@@ -2,11 +2,12 @@
 '''
 Manage and query NPM packages.
 '''
+from __future__ import absolute_import
 
 # Import python libs
 import json
 import logging
-import distutils.version  # pylint: disable=E0611
+import distutils.version  # pylint: disable=import-error,no-name-in-module
 
 # Import salt libs
 import salt.utils
@@ -25,25 +26,41 @@ def __virtual__():
     '''
     Only work when npm is installed.
     '''
-    if salt.utils.which('npm'):
-        return 'npm'
-    return False
+    try:
+        if salt.utils.which('npm') is not None:
+            _check_valid_version()
+            return True
+        else:
+            return (False, 'npm execution module could not be loaded '
+                           'because the npm binary could not be located')
+    except CommandExecutionError as exc:
+        return (False, str(exc))
 
 
-def _valid_version():
+def _check_valid_version():
     '''
     Check the version of npm to ensure this module will work. Currently
     npm must be at least version 1.2.
     '''
+    # pylint: disable=no-member
     npm_version = distutils.version.LooseVersion(
         __salt__['cmd.run']('npm --version'))
     valid_version = distutils.version.LooseVersion('1.2')
-    return npm_version >= valid_version
+    # pylint: enable=no-member
+    if npm_version < valid_version:
+        raise CommandExecutionError(
+            '\'npm\' is not recent enough({0} < {1}). Please Upgrade.'.format(
+                npm_version, valid_version
+            )
+        )
 
 
 def install(pkg=None,
+            pkgs=None,
             dir=None,
-            runas=None):
+            runas=None,
+            registry=None,
+            env=None):
     '''
     Install an NPM package.
 
@@ -52,7 +69,13 @@ def install(pkg=None,
     package in the given directory will be installed.
 
     pkg
-        A package name in any format accepted by NPM
+        A package name in any format accepted by NPM, including a version
+        identifier
+
+    pkgs
+        A list of package names in the same format as the ``name`` parameter
+
+        .. versionadded:: 2014.7.0
 
     dir
         The target directory in which to install the package, or None for
@@ -61,25 +84,42 @@ def install(pkg=None,
     runas
         The user to run NPM with
 
+    registry
+        The NPM registry to install the package from.
+
+        .. versionadded:: 2014.7.0
+
+    env
+        Environment variables to set when invoking npm. Uses the same ``env``
+        format as the :py:func:`cmd.run <salt.modules.cmdmod.run>` execution
+        function.
+
+        .. versionadded:: 2014.7.0
+
     CLI Example:
 
     .. code-block:: bash
 
         salt '*' npm.install coffee-script
 
+        salt '*' npm.install coffee-script@1.0.1
+
     '''
-    if not _valid_version():
-        return '{0!r} is not available.'.format('npm.install')
 
     cmd = 'npm install --silent --json'
 
     if dir is None:
         cmd += ' --global'
 
+    if registry:
+        cmd += ' --registry="{0}"'.format(registry)
+
     if pkg:
         cmd += ' "{0}"'.format(pkg)
+    elif pkgs:
+        cmd += ' "{0}"'.format('" "'.join(pkgs))
 
-    result = __salt__['cmd.run_all'](cmd, cwd=dir, runas=runas)
+    result = __salt__['cmd.run_all'](cmd, python_shell=False, cwd=dir, runas=runas, env=env)
 
     if result['retcode'] != 0:
         raise CommandExecutionError(result['stderr'])
@@ -95,15 +135,18 @@ def install(pkg=None,
     lines = npm_output.splitlines()
     log.error(lines)
 
-    # Strip all lines until JSON output starts
-    while not lines[0].startswith('{') and not lines[0].startswith('['):
-        lines = lines[1:]
+    while lines:
+        # Strip all lines until JSON output starts
+        while not lines[0].startswith('{') and not lines[0].startswith('['):
+            lines = lines[1:]
 
-    try:
-        return json.loads(''.join(lines))
-    except ValueError:
-        # Still no JSON!! Return the stdout as a string
-        return npm_output
+        try:
+            return json.loads(''.join(lines))
+        except ValueError:
+            lines = lines[1:]
+
+    # Still no JSON!! Return the stdout as a string
+    return npm_output
 
 
 def uninstall(pkg,
@@ -131,9 +174,6 @@ def uninstall(pkg,
         salt '*' npm.uninstall coffee-script
 
     '''
-    if not _valid_version():
-        log.error('{0!r} is not available.'.format('npm.uninstall'))
-        return False
 
     cmd = 'npm uninstall'
 
@@ -142,7 +182,7 @@ def uninstall(pkg,
 
     cmd += ' "{0}"'.format(pkg)
 
-    result = __salt__['cmd.run_all'](cmd, cwd=dir, runas=runas)
+    result = __salt__['cmd.run_all'](cmd, python_shell=False, cwd=dir, runas=runas)
 
     if result['retcode'] != 0:
         log.error(result['stderr'])
@@ -150,7 +190,10 @@ def uninstall(pkg,
     return True
 
 
-def list_(pkg=None, dir=None):
+def list_(pkg=None,
+            dir=None,
+            runas=None,
+            env=None):
     '''
     List installed NPM packages.
 
@@ -164,6 +207,18 @@ def list_(pkg=None, dir=None):
         The directory whose packages will be listed, or None for global
         installation
 
+    runas
+        The user to run NPM with
+
+        .. versionadded:: 2014.7.0
+
+    env
+        Environment variables to set when invoking npm. Uses the same ``env``
+        format as the :py:func:`cmd.run <salt.modules.cmdmod.run>` execution
+        function.
+
+        .. versionadded:: 2014.7.0
+
     CLI Example:
 
     .. code-block:: bash
@@ -171,10 +226,8 @@ def list_(pkg=None, dir=None):
         salt '*' npm.list
 
     '''
-    if not _valid_version():
-        return '{0!r} is not available.'.format('npm.list')
 
-    cmd = 'npm list --json'
+    cmd = 'npm list --silent --json'
 
     if dir is None:
         cmd += ' --global'
@@ -182,9 +235,17 @@ def list_(pkg=None, dir=None):
     if pkg:
         cmd += ' "{0}"'.format(pkg)
 
-    result = __salt__['cmd.run_all'](cmd, cwd=dir)
+    result = __salt__['cmd.run_all'](
+            cmd,
+            cwd=dir,
+            runas=runas,
+            env=env,
+            python_shell=False,
+            ignore_retcode=True)
 
-    if result['retcode'] != 0:
+    # npm will return error code 1 for both no packages found and an actual
+    # error. The only difference between the two cases are if stderr is empty
+    if result['retcode'] != 0 and result['stderr']:
         raise CommandExecutionError(result['stderr'])
 
     return json.loads(result['stdout']).get('dependencies', {})
