@@ -4,6 +4,7 @@ Configure ``portage(5)``
 '''
 
 # Import python libs
+from __future__ import absolute_import
 import os
 import shutil
 
@@ -49,7 +50,7 @@ def _porttree():
 def _p_to_cp(p):
     '''
     Convert a package name or a DEPEND atom to category/package format.
-    Raises an exception if program name is ambigous.
+    Raises an exception if program name is ambiguous.
     '''
     ret = _porttree().dbapi.xmatch("match-all", p)
     if ret:
@@ -133,7 +134,7 @@ def _package_conf_file_to_dir(file_name):
                 return False
             else:
                 os.rename(path, path + '.tmpbak')
-                os.mkdir(path, 0755)
+                os.mkdir(path, 0o755)
                 with salt.utils.fopen(path + '.tmpbak') as fh_:
                     for line in fh_:
                         line = line.strip()
@@ -142,7 +143,7 @@ def _package_conf_file_to_dir(file_name):
                 os.remove(path + '.tmpbak')
                 return True
         else:
-            os.mkdir(path, 0755)
+            os.mkdir(path, 0o755)
             return True
 
 
@@ -237,7 +238,7 @@ def _merge_flags(new_flags, old_flags=None, conf='any'):
         else:
             flags[flag] = True
     tmp = []
-    for k, v in flags.iteritems():
+    for k, v in flags.items():
         if v:
             tmp.append(k)
         else:
@@ -304,7 +305,7 @@ def append_to_package_conf(conf, atom='', flags=None, string='', overwrite=False
         if len(psplit) == 2:
             pdir = BASE_PATH.format(conf) + '/' + psplit[0]
             if not os.path.exists(pdir):
-                os.mkdir(pdir, 0755)
+                os.mkdir(pdir, 0o755)
 
         complete_file_path = BASE_PATH.format(conf) + '/' + package_file
 
