@@ -4,6 +4,7 @@ Module to provide RabbitMQ compatibility to Salt.
 Todo: A lot, need to add cluster support, logging, and minion configuration
 data.
 '''
+from __future__ import absolute_import
 
 # Import salt libs
 import salt.utils
@@ -12,6 +13,7 @@ import salt.utils
 import logging
 import random
 import string
+from salt.ext.six.moves import range
 
 log = logging.getLogger(__name__)
 
@@ -55,24 +57,6 @@ def _get_rabbitmq_plugin():
     return rabbitmq
 
 
-def _output_to_dict(cmdoutput, values_mapper=None):
-    '''Convert rabbitmqctl output to a dict of data
-    cmdoutput: string output of rabbitmqctl commands
-    values_mapper: function object to process the values part of each line
-    '''
-    ret = {}
-    if values_mapper is None:
-        values_mapper = lambda string: string.split('\t')
-
-    # remove first and last line: Listing ... - ...done
-    data_rows = cmdoutput.splitlines()[1:-1]
-    for row in data_rows:
-        key, values = row.split('\t', 1)
-        ret[key] = values_mapper(values)
-
-    return ret
-
-
 def list_users(runas=None):
     '''
     Return a list of users based off of rabbitmqctl user_list.
@@ -83,14 +67,19 @@ def list_users(runas=None):
 
         salt '*' rabbitmq.list_users
     '''
+    ret = {}
     if runas is None:
         runas = salt.utils.get_user()
     res = __salt__['cmd.run']('rabbitmqctl list_users',
                               runas=runas)
-
-    # func to get tags from string such as "[admin, monitoring]"
-    func = lambda string: set(string[1:-1].split(','))
-    return _output_to_dict(res, func)
+    for line in res.splitlines():
+        if '...' not in line or line == '\n':
+            parts = line.split('\t')
+            if len(parts) < 2:
+                continue
+            user, properties = parts[0], parts[1]
+            ret[user] = properties
+    return ret
 
 
 def list_vhosts(runas=None):
@@ -107,9 +96,9 @@ def list_vhosts(runas=None):
         runas = salt.utils.get_user()
     res = __salt__['cmd.run']('rabbitmqctl list_vhosts',
                               runas=runas)
-
-    # remove first and last line: Listing ... - ...done
-    return res.splitlines()[1:-1]
+    lines = res.splitlines()
+    vhost_list = [line for line in lines if '...' not in line]
+    return vhost_list
 
 
 def user_exists(name, runas=None):
@@ -324,8 +313,7 @@ def list_permissions(vhost, runas=None):
         'rabbitmqctl list_permissions -p {0}'.format(vhost),
         python_shell=False,
         runas=runas)
-
-    return _output_to_dict(res)
+    return [r.split('\t') for r in res.splitlines()]
 
 
 def list_user_permissions(name, runas=None):
@@ -344,8 +332,7 @@ def list_user_permissions(name, runas=None):
         'rabbitmqctl list_user_permissions {0}'.format(name),
         python_shell=False,
         runas=runas)
-
-    return _output_to_dict(res)
+    return [r.split('\t') for r in res.splitlines()]
 
 
 def set_user_tags(name, tags, runas=None):
@@ -359,10 +346,6 @@ def set_user_tags(name, tags, runas=None):
     '''
     if runas is None:
         runas = salt.utils.get_user()
-
-    if tags and isinstance(tags, (list, tuple)):
-        tags = ' '.join(tags)
-
     res = __salt__['cmd.run'](
         'rabbitmqctl set_user_tags {0} {1}'.format(name, tags),
         python_shell=False,
@@ -409,7 +392,7 @@ def cluster_status(runas=None):
     return res
 
 
-def join_cluster(host, user='rabbit', runas=None):
+def join_cluster(host, user='rabbit', ram_node=None, runas=None):
     '''
     Join a rabbit cluster
 
@@ -417,15 +400,17 @@ def join_cluster(host, user='rabbit', runas=None):
 
     .. code-block:: bash
 
-        salt '*' rabbitmq.join_cluster 'rabbit' 'rabbit.example.com'
+        salt '*' rabbitmq.join_cluster 'rabbit.example.com' 'rabbit'
     '''
+    if ram_node:
+        cmd = 'rabbitmqctl join_cluster --ram {0}@{1}'.format(user, host)
+    else:
+        cmd = 'rabbitmqctl join_cluster {0}@{1}'.format(user, host)
+
     if runas is None:
         runas = salt.utils.get_user()
     stop_app(runas)
-    res = __salt__['cmd.run'](
-        'rabbitmqctl join_cluster {0}@{1}'.format(user, host),
-        python_shell=False,
-        runas=runas)
+    res = __salt__['cmd.run'](cmd, runas=runas, python_shell=False)
     start_app(runas)
 
     return _format_response(res, 'Join')
@@ -574,17 +559,21 @@ def list_policies(runas=None):
     for line in res.splitlines():
         if '...' not in line and line != '\n':
             parts = line.split('\t')
-            if len(parts) != 6:
+            if len(parts) not in (5, 6):
                 continue
             vhost, name = parts[0], parts[1]
             if vhost not in ret:
                 ret[vhost] = {}
-            ret[vhost][name] = {
-                'apply_to': parts[2],
-                'pattern': parts[3],
-                'definition': parts[4],
-                'priority': parts[5]
-            }
+            ret[vhost][name] = {}
+            # How many fields are there? - 'apply_to' was inserted in position 2 at somepoint
+            offset = len(parts) - 5
+            if len(parts) == 6:
+                ret[vhost][name]['apply_to'] = parts[2]
+            ret[vhost][name].update({
+                'pattern': parts[offset+2],
+                'definition': parts[offset+3],
+                'priority': parts[offset+4]
+            })
     log.debug('Listing policies: {0}'.format(ret))
     return ret
 
