@@ -5,12 +5,16 @@ Wrapper around Server Density API
 
 .. versionadded:: 2014.7.0
 '''
+from __future__ import absolute_import
+
 import requests
 import json
 import logging
 import os
 import tempfile
 
+from salt.ext.six.moves import map
+
 from salt.exceptions import CommandExecutionError
 
 log = logging.getLogger(__name__)
@@ -45,7 +49,7 @@ def _clean_salt_variables(params, variable_prefix="__"):
     '''
     Pops out variables from params which starts with `variable_prefix`.
     '''
-    map(params.pop, [k for k in params if k.startswith(variable_prefix)])
+    list(list(map(params.pop, [k for k in params if k.startswith(variable_prefix)])))
     return params
 
 
