@@ -41,7 +41,7 @@ To generate a cipher from a secret:
 
 .. code-block:: bash
 
-   $ echo -n"supersecret" | gpg --homedir --armor --encrypt -r <KEY-name>
+   $ echo -n "supersecret" | gpg --homedir --armor --encrypt -r <KEY-name>
 
 Set up the renderer on your master by adding something like this line to your
 config:
@@ -69,6 +69,7 @@ Now you can include your ciphers in your pillar data like so:
       =Eqsm
       -----END PGP MESSAGE-----
 '''
+from __future__ import absolute_import
 
 import os
 import re
@@ -135,9 +136,10 @@ def render(gpg_data, saltenv='base', sls='', argline='', **kwargs):
     '''
     if not HAS_GPG:
         raise SaltRenderError('GPG unavailable')
+    homedir = None
     if 'config.get' in __salt__:
-        homedir = __salt__['config.get']('gpg_keydir', DEFAULT_GPG_KEYDIR)
-    else:
+        homedir = __salt__['config.get']('gpg_keydir', None)
+    if homedir is None:
         homedir = __opts__.get('gpg_keydir', DEFAULT_GPG_KEYDIR)
     log.debug('Reading GPG keys from: {0}'.format(homedir))
     try:
