@@ -2,12 +2,14 @@
 '''
 Directly manage the Salt fileserver plugins
 '''
+from __future__ import absolute_import
 
 # Import Salt libs
+import salt.utils
 import salt.fileserver
 
 
-def envs(backend=None, sources=False, outputter='nested'):
+def envs(backend=None, sources=False, outputter=None):
     '''
     Return the available fileserver environments. If no backend is provided,
     then the environments for all configured backends will be returned.
@@ -15,7 +17,7 @@ def envs(backend=None, sources=False, outputter='nested'):
     backend
         Narrow fileserver backends to a subset of the enabled ones.
 
-        .. versionchanged:: 2015.5.0
+        .. versionchanged:: 2015.2.0::
             If all passed backends start with a minus sign (``-``), then these
             backends will be excluded from the enabled backends. However, if
             there is a mix of backends with and without a minus sign (ex:
@@ -31,7 +33,6 @@ def envs(backend=None, sources=False, outputter='nested'):
     .. code-block:: bash
 
         salt-run fileserver.envs
-        salt-run fileserver.envs outputter=nested
         salt-run fileserver.envs backend=roots,git
         salt-run fileserver.envs git
     '''
@@ -39,11 +40,18 @@ def envs(backend=None, sources=False, outputter='nested'):
     output = fileserver.envs(back=backend, sources=sources)
 
     if outputter:
-        salt.output.display_output(output, outputter, opts=__opts__)
-    return output
+        salt.utils.warn_until(
+            'Boron',
+            'The \'outputter\' argument to the fileserver.envs runner has '
+            'been deprecated. Please specify an outputter using --out. '
+            'See the output of \'salt-run -h\' for more information.'
+        )
+        return {'outputter': outputter, 'data': output}
+    else:
+        return output
 
 
-def file_list(saltenv='base', backend=None, outputter='nested'):
+def file_list(saltenv='base', backend=None, outputter=None):
     '''
     Return a list of files from the salt fileserver
 
@@ -58,7 +66,7 @@ def file_list(saltenv='base', backend=None, outputter='nested'):
         ``backend=-roots,git``) then the ones starting with a minus sign will
         be disregarded.
 
-        .. versionadded:: 2015.5.0
+        .. versionadded:: 2015.2.0
 
     CLI Examples:
 
@@ -75,11 +83,18 @@ def file_list(saltenv='base', backend=None, outputter='nested'):
     output = fileserver.file_list(load=load)
 
     if outputter:
-        salt.output.display_output(output, outputter, opts=__opts__)
-    return output
+        salt.utils.warn_until(
+            'Boron',
+            'The \'outputter\' argument to the fileserver.file_list runner '
+            'has been deprecated. Please specify an outputter using --out. '
+            'See the output of \'salt-run -h\' for more information.'
+        )
+        return {'outputter': outputter, 'data': output}
+    else:
+        return output
 
 
-def symlink_list(saltenv='base', backend=None, outputter='nested'):
+def symlink_list(saltenv='base', backend=None, outputter=None):
     '''
     Return a list of symlinked files and dirs
 
@@ -94,7 +109,7 @@ def symlink_list(saltenv='base', backend=None, outputter='nested'):
         ``backend=-roots,git``) then the ones starting with a minus sign will
         be disregarded.
 
-        .. versionadded:: 2015.5.0
+        .. versionadded:: 2015.2.0
 
     CLI Example:
 
@@ -111,11 +126,18 @@ def symlink_list(saltenv='base', backend=None, outputter='nested'):
     output = fileserver.symlink_list(load=load)
 
     if outputter:
-        salt.output.display_output(output, outputter, opts=__opts__)
-    return output
+        salt.utils.warn_until(
+            'Boron',
+            'The \'outputter\' argument to the fileserver.symlink_list '
+            'runner has been deprecated. Please specify an outputter using '
+            '--out. See the output of \'salt-run -h\' for more information.'
+        )
+        return {'outputter': outputter, 'data': output}
+    else:
+        return output
 
 
-def dir_list(saltenv='base', backend=None, outputter='nested'):
+def dir_list(saltenv='base', backend=None, outputter=None):
     '''
     Return a list of directories in the given environment
 
@@ -130,7 +152,7 @@ def dir_list(saltenv='base', backend=None, outputter='nested'):
         ``backend=-roots,git``) then the ones starting with a minus sign will
         be disregarded.
 
-        .. versionadded:: 2015.5.0
+        .. versionadded:: 2015.2.0
 
     CLI Example:
 
@@ -147,13 +169,20 @@ def dir_list(saltenv='base', backend=None, outputter='nested'):
     output = fileserver.dir_list(load=load)
 
     if outputter:
-        salt.output.display_output(output, outputter, opts=__opts__)
-    return output
+        salt.utils.warn_until(
+            'Boron',
+            'The \'outputter\' argument to the fileserver.dir_list runner '
+            'has been deprecated. Please specify an outputter using --out. '
+            'See the output of \'salt-run -h\' for more information.'
+        )
+        return {'outputter': outputter, 'data': output}
+    else:
+        return output
 
 
-def empty_dir_list(saltenv='base', backend=None, outputter='nested'):
+def empty_dir_list(saltenv='base', backend=None, outputter=None):
     '''
-    .. versionadded:: 2015.5.0
+    .. versionadded:: 2015.2.0
 
     Return a list of empty directories in the given environment
 
@@ -188,8 +217,15 @@ def empty_dir_list(saltenv='base', backend=None, outputter='nested'):
     output = fileserver.file_list_emptydirs(load=load)
 
     if outputter:
-        salt.output.display_output(output, outputter, opts=__opts__)
-    return output
+        salt.utils.warn_until(
+            'Boron',
+            'The \'outputter\' argument to the fileserver.empty_dir_list '
+            'runner has been deprecated. Please specify an outputter using '
+            '--out. See the output of \'salt-run -h\' for more information.'
+        )
+        return {'outputter': outputter, 'data': output}
+    else:
+        return output
 
 
 def update(backend=None):
@@ -200,7 +236,7 @@ def update(backend=None):
     backend
         Narrow fileserver backends to a subset of the enabled ones.
 
-        .. versionchanged:: 2015.5.0
+        .. versionchanged:: 2015.2.0
             If all passed backends start with a minus sign (``-``), then these
             backends will be excluded from the enabled backends. However, if
             there is a mix of backends with and without a minus sign (ex:
@@ -225,7 +261,7 @@ def update(backend=None):
 
 def clear_cache(backend=None):
     '''
-    .. versionadded:: 2015.5.0
+    .. versionadded:: 2015.2.0
 
     Clear the fileserver cache from VCS fileserver backends (:mod:`git
     <salt.fileserver.gitfs>`, :mod:`hg <salt.fileserver.hgfs>`, :mod:`svn
@@ -263,7 +299,7 @@ def clear_cache(backend=None):
 
 def clear_lock(backend=None, remote=None):
     '''
-    .. versionadded:: 2015.5.0
+    .. versionadded:: 2015.2.0
 
     Clear the fileserver update lock from VCS fileserver backends (:mod:`git
     <salt.fileserver.gitfs>`, :mod:`hg <salt.fileserver.hgfs>`, :mod:`svn
@@ -304,7 +340,7 @@ def clear_lock(backend=None, remote=None):
 
 def lock(backend=None, remote=None):
     '''
-    .. versionadded:: 2015.5.0
+    .. versionadded:: 2015.2.0
 
     Set a fileserver update lock for VCS fileserver backends (:mod:`git
     <salt.fileserver.gitfs>`, :mod:`hg <salt.fileserver.hgfs>`, :mod:`svn
