@@ -13,9 +13,10 @@ Use this minion to spin up a cloud instance:
       cloud.profile:
         my-ec2-config
 '''
+from __future__ import absolute_import
 
 import pprint
-from salt._compat import string_types
+from salt.ext.six import string_types
 import salt.utils.cloud as suc
 
 
@@ -49,15 +50,6 @@ def _valid(name, comment='', changes=None):
             'comment': comment}
 
 
-def _get_instance(names):
-    # for some reason loader overwrites __opts__['test'] with default
-    # value of False, thus store and then load it again after action
-    test = __opts__['test']
-    instance = __salt__['cloud.action'](fun='show_instance', names=names)
-    __opts__['test'] = test
-    return instance
-
-
 def present(name, cloud_provider, onlyif=None, unless=None, **kwargs):
     '''
     Spin up a single instance on a cloud provider, using salt-cloud. This state
@@ -86,9 +78,8 @@ def present(name, cloud_provider, onlyif=None, unless=None, **kwargs):
            'changes': {},
            'result': None,
            'comment': ''}
-    instance = _get_instance([name])
+
     retcode = __salt__['cmd.retcode']
-    prov = str([a for a in instance][0])
     if onlyif is not None:
         if not isinstance(onlyif, string_types):
             if not onlyif:
@@ -103,14 +94,18 @@ def present(name, cloud_provider, onlyif=None, unless=None, **kwargs):
         elif isinstance(unless, string_types):
             if retcode(unless) == 0:
                 return _valid(name, comment='unless execution succeeded')
-    if instance and 'Not Actioned' not in prov:
+
+    # provider=None not cloud_provider because
+    # need to ensure ALL providers don't have the instance
+    if __salt__['cloud.has_instance'](name=name, provider=None):
         ret['result'] = True
-        ret['comment'] = 'Instance {0} already exists in {1}'.format(name,
-                                                                     prov)
+        ret['comment'] = 'Already present instance {0}'.format(name)
         return ret
+
     if __opts__['test']:
         ret['comment'] = 'Instance {0} needs to be created'.format(name)
         return ret
+
     info = __salt__['cloud.create'](cloud_provider, name, **kwargs)
     if info and 'Error' not in info:
         ret['changes'] = info
@@ -161,16 +156,7 @@ def absent(name, onlyif=None, unless=None):
            'result': None,
            'comment': ''}
     retcode = __salt__['cmd.retcode']
-    instance = _get_instance([name])
-    if not instance or \
-            ('Not Actioned/Not Running' in instance
-            and name in instance['Not Actioned/Not Running']):
-        ret['result'] = True
-        ret['comment'] = 'Instance {0} already absent'.format(name)
-        return ret
-    if __opts__['test']:
-        ret['comment'] = 'Instance {0} needs to be destroyed'.format(name)
-        return ret
+
     if onlyif is not None:
         if not isinstance(onlyif, string_types):
             if not onlyif:
@@ -185,6 +171,16 @@ def absent(name, onlyif=None, unless=None):
         elif isinstance(unless, string_types):
             if retcode(unless) == 0:
                 return _valid(name, comment='unless execution succeeded')
+
+    if not __salt__['cloud.has_instance'](name=name, provider=None):
+        ret['result'] = True
+        ret['comment'] = 'Already absent instance {0}'.format(name)
+        return ret
+
+    if __opts__['test']:
+        ret['comment'] = 'Instance {0} needs to be destroyed'.format(name)
+        return ret
+
     info = __salt__['cloud.destroy'](name)
     if info and 'Error' not in info:
         ret['changes'] = info
@@ -249,16 +245,17 @@ def profile(name, profile, onlyif=None, unless=None, **kwargs):
         elif isinstance(unless, string_types):
             if retcode(unless) == 0:
                 return _valid(name, comment='unless execution succeeded')
-    instance = _get_instance([name])
-    prov = str(instance.iterkeys().next())
+    instance = __salt__['cloud.action'](fun='show_instance', names=[name])
+    prov = str(next(instance.iterkeys()))
     if instance and 'Not Actioned' not in prov:
         ret['result'] = True
-        ret['comment'] = 'Instance {0} already exists in {1}'.format(
-            name, prov)
+        ret['comment'] = 'Already present instance {0}'.format(name)
         return ret
+
     if __opts__['test']:
         ret['comment'] = 'Instance {0} needs to be created'.format(name)
         return ret
+
     info = __salt__['cloud.profile'](profile, name, vm_overrides=kwargs)
 
     # get either {Error: ''} or {namestring: {Error: ''}}
@@ -388,8 +385,9 @@ def volume_attached(name, server_name, provider=None, **kwargs):
 
     if name in volumes and volumes[name]['attachments']:
         volume = volumes[name]
-        ret['comment'] = ('Volume {name} is already'
-                          'attached: {attachments}').format(**volumes[name])
+        ret['comment'] = (
+                          'Volume {name} is already attached: {attachments}'
+                          ).format(**volumes[name])
         ret['result'] = True
         return ret
     elif name not in volumes:
