@@ -57,6 +57,16 @@ Available Functions
           - name: mysuperdocker
           - hostname: superdocker
           - image: corp/mysuperdocker_img
+
+- loaded
+
+  .. code-block:: yaml
+
+      mysuperdocker-file:
+        docker.loaded:
+          - name: mysuperdocker
+          - source: salt://_files/tmp/docker_image.tar
+
 - running
 
   .. code-block:: yaml
@@ -71,12 +81,11 @@ Available Functions
 
   .. note::
 
-      The ``port_bindings`` argument above is a dictionary. Note the
-      double-indentation, this is required for PyYAML to load the data
-      structure properly as a dictionary. More information can be found
+      The ``port_bindings`` argument above is a dictionary. The double
+      indentation is required for PyYAML to load the data structure
+      properly as a python dictionary. More information can be found
       :ref:`here <nested-dict-indentation>`
 
-
 - absent
 
   .. code-block:: yaml
@@ -99,22 +108,17 @@ Available Functions
     The docker modules are named ``dockerio`` because
     the name 'docker' would conflict with the underlying docker-py library.
 
-    We should add magic to all methods to also match containers by name
-    now that the 'naming link' stuff has been merged in docker.
-    This applies for example to:
-
-    - running
-    - absent
-    - run
-
 
 '''
+
+from __future__ import absolute_import
 import functools
 import logging
 
 # Import salt libs
-from salt._compat import string_types
+from salt.ext.six import string_types
 import salt.utils
+import salt.ext.six as six
 
 # Enable proper logging
 log = logging.getLogger(__name__)
@@ -220,6 +224,7 @@ def mod_watch(name, sfun=None, *args, **kw):
 def pulled(name,
            tag='latest',
            force=False,
+           insecure_registry=False,
            *args,
            **kwargs):
     '''
@@ -233,9 +238,9 @@ def pulled(name,
         (`docker import
         <http://docs.docker.io/en/latest/reference/commandline/cli/#import>`_).
         NOTE that we added in SaltStack a way to authenticate yourself with the
-        Docker Hub Registry by supplying your credentials (username, email & password)
-        using pillars. For more information, see salt.modules.dockerio execution
-        module.
+        Docker Hub Registry by supplying your credentials (username, email &
+        password) using pillars. For more information, see salt.modules.dockerio
+        execution module.
 
     name
         Name of the image
@@ -245,6 +250,9 @@ def pulled(name,
 
     force
         Pull even if the image is already pulled
+
+    insecure_registry
+        Set to ``True`` to allow connections to non-HTTPS registries. Default ``False``.
     '''
 
     inspect_image = __salt__['docker.inspect_image']
@@ -260,7 +268,7 @@ def pulled(name,
 
     previous_id = image_infos['out']['Id'] if image_infos['status'] else None
     pull = __salt__['docker.pull']
-    returned = pull(name, tag=tag)
+    returned = pull(name, tag=tag, insecure_registry=insecure_registry)
     if previous_id != returned['id']:
         changes = {name: {'old': previous_id,
                           'new': returned['id']}}
@@ -269,7 +277,7 @@ def pulled(name,
     return _ret_status(returned, name, changes=changes)
 
 
-def pushed(name, tag='latest'):
+def pushed(name, tag='latest', insecure_registry=False):
     '''
     Push an image from a docker registry. (`docker push`)
 
@@ -281,9 +289,9 @@ def pushed(name, tag='latest'):
         (`docker import
         <http://docs.docker.io/en/latest/reference/commandline/cli/#import>`_).
         NOTE that we added in SaltStack a way to authenticate yourself with the
-        Docker Hub Registry by supplying your credentials (username, email & password)
-        using pillars. For more information, see salt.modules.dockerio execution
-        module.
+        Docker Hub Registry by supplying your credentials (username, email
+        & password) using pillars. For more information, see
+        salt.modules.dockerio execution module.
 
     name
         Name of the image
@@ -291,6 +299,8 @@ def pushed(name, tag='latest'):
     tag
         Tag of the image [Optional]
 
+    insecure_registry
+        Set to ``True`` to allow connections to non-HTTPS registries. Default ``False``.
     '''
 
     if __opts__['test']:
@@ -298,7 +308,7 @@ def pushed(name, tag='latest'):
         return _ret_status(name=name, comment=comment)
 
     push = __salt__['docker.push']
-    returned = push(name, tag=tag)
+    returned = push(name, tag=tag, insecure_registry=insecure_registry)
     log.debug("Returned: "+str(returned))
     if returned['status']:
         changes = {name: {'Rev': returned['id']}}
@@ -307,6 +317,77 @@ def pushed(name, tag='latest'):
     return _ret_status(returned, name, changes=changes)
 
 
+def loaded(name, source=None, source_hash='', force=False):
+    '''
+    Load an image into the local docker registry (`docker load`)
+
+    name
+        Name of the docker image
+
+    source
+        The source .tar file to download to the minion, created by docker save
+        this source file can be hosted on either the salt master server,
+        or on an HTTP or FTP server.
+
+        If the file is hosted on a HTTP or FTP server then the source_hash
+        argument is also required
+
+        .. note::
+
+            See first the documentation for salt file.managed
+            <http://docs.saltstack.com/en/latest/ref/states/all/_
+            salt.states.file.html#salt.states.file.managed>
+
+    source_hash
+        This can be one of the following:
+            1. a source hash string
+            2. the URI of a file that contains source hash strings
+
+            .. note::
+
+                See first the documentation for salt file.managed
+                <http://docs.saltstack.com/en/latest/ref/states/all/_
+                salt.states.file.html#salt.states.file.managed>
+
+    force
+        Load even if the image exists
+    '''
+
+    inspect_image = __salt__['docker.inspect_image']
+    image_infos = inspect_image(name)
+    if image_infos['status'] and not force:
+        return _valid(
+            name=name,
+            comment='Image already loaded: {0}'.format(name))
+
+    tmp_filename = salt.utils.mkstemp()
+    __salt__['state.single']('file.managed',
+                            name=tmp_filename,
+                            source=source,
+                            source_hash=source_hash)
+    changes = {}
+
+    if image_infos['status']:
+        changes['old'] = image_infos['out']['Id']
+        remove_image = __salt__['docker.remove_image']
+        remove_info = remove_image(name)
+        if not remove_info['status']:
+            return _invalid(name=name,
+                    comment='Image could not be removed: {0}'.format(name))
+
+    load = __salt__['docker.load']
+    returned = load(tmp_filename)
+
+    image_infos = inspect_image(name)
+    if image_infos['status']:
+        changes['new'] = image_infos['out']['Id']
+    else:
+        return _invalid(name=name,
+                        comment='Image {0} was not loaded into docker'.format(name))
+
+    return _ret_status(returned, name, changes=changes)
+
+
 def built(name,
           path=None,
           quiet=False,
@@ -375,6 +456,8 @@ def installed(name,
               dns=None,
               volumes=None,
               volumes_from=None,
+              cpu_shares=None,
+              cpuset=None,
               *args, **kwargs):
     '''
     Ensure that a container with the given name exists;
@@ -426,12 +509,12 @@ def installed(name,
         volumes = []
     if isinstance(environment, dict):
         for k in environment:
-            denvironment[unicode(k)] = unicode(environment[k])
+            denvironment[six.text_type(k)] = six.text_type(environment[k])
     if isinstance(environment, list):
         for p in environment:
             if isinstance(p, dict):
                 for k in p:
-                    denvironment[unicode(k)] = unicode(p[k])
+                    denvironment[six.text_type(k)] = six.text_type(p[k])
     for p in ports:
         if not isinstance(p, dict):
             dports[str(p)] = {}
@@ -459,14 +542,16 @@ def installed(name,
         dns=dns,
         volumes=dvolumes,
         volumes_from=volumes_from,
-        name=name)
+        name=name,
+        cpu_shares=cpu_shares,
+        cpuset=cpuset)
     out = create(*a, **kw)
     # if container has been created, even if not started, we mark
     # it as installed
     changes = 'Container created'
     try:
         cid = out['out']['info']['id']
-    except Exception, e:
+    except Exception as e:
         log.debug(str(e))
     else:
         changes = 'Container {0} created'.format(cid)
@@ -485,17 +570,21 @@ def absent(name):
     '''
     ins_container = __salt__['docker.inspect_container']
     cinfos = ins_container(name)
+    changes = {}
+
     if cinfos['status']:
         cid = cinfos['id']
+        changes[cid] = {}
         is_running = __salt__['docker.is_running'](cid)
-        # destroy if we found meat to do
+
+        # Stop container gracefully, if running
         if is_running:
+            changes[cid]['old'] = 'running'
             __salt__['docker.stop'](cid)
             is_running = __salt__['docker.is_running'](cid)
             if is_running:
-                return _invalid(
-                    comment=('Container {0!r}'
-                             ' could not be stopped'.format(cid)))
+                return _invalid(comment=("Container {0!r} could not be stopped"
+                                         .format(cid)))
             else:
                 __salt__['docker.remove_container'](cid)
                 is_gone = __salt__['docker.exists'](cid)
@@ -519,16 +608,22 @@ def absent(name):
                                        ' is stopped but could not be destroyed,'.format(cid)),
                                        changes={name: True})
     else:
-        return _valid(comment='Container {0!r} not found'.format(name))
+        return _valid(comment="Container {0!r} not found".format(name))
 
 
-def present(name):
+def present(name, image=None, is_latest=False):
     '''
     If a container with the given name is not present, this state will fail.
+    Supports optionally checking for specific image/version
     (`docker inspect`)
 
     name:
         container id
+    image:
+        image the container should be running (defaults to any)
+    is_latest:
+        also check if the container runs the latest version of the image (
+        latest defined as the latest pulled onto the local machine)
     '''
     ins_container = __salt__['docker.inspect_container']
     cinfos = ins_container(name)
@@ -536,10 +631,17 @@ def present(name):
         cid = cinfos['id']
     else:
         cid = name
-    if cinfos['status']:
-        return _valid(comment='Container {0} exists'.format(cid))
-    else:
+    if not cinfos['status']:
         return _invalid(comment='Container {0} not found'.format(cid or name))
+    if cinfos['status'] and image is None:
+        return _valid(comment='Container {0} exists'.format(cid))
+    if cinfos['status'] and cinfos['out']['Config']["Image"] == image and not is_latest:
+        return _valid(comment='Container {0} exists and has image {1}'.format(cid, image))
+    ins_image = __salt__['docker.inspect_image']
+    iinfos = ins_image(image)
+    if cinfos['status'] and cinfos['out']['Image'] == iinfos['out']['Id']:
+        return _valid(comment='Container {0} exists and has latest version of image {1}'.format(cid, image))
+    return _invalid(comment='Container {0} found with wrong image'.format(cid or name))
 
 
 def run(name,
@@ -559,7 +661,7 @@ def run(name,
         command to run in the container
 
     cid
-        Container id
+        Container id or name
 
     state_id
         state_id
@@ -636,63 +738,108 @@ def script(*args, **kw):
     raise NotImplementedError
 
 
-def running(name, container=None, port_bindings=None, binds=None,
-            publish_all_ports=False, links=None, lxc_conf=None,
-            privileged=False, dns=None, volumes_from=None,
-            network_mode=None, restart_policy=None, cap_add=None,
-            cap_drop=None, check_is_running=True):
+def running(name,
+            image,
+            container=None,
+            command=None,
+            hostname=None,
+            user=None,
+            detach=True,
+            stdin_open=False,
+            tty=False,
+            mem_limit=0,
+            ports=None,
+            environment=None,
+            dns=None,
+            volumes=None,
+            volumes_from=None,
+            start=True,
+            cap_add=None,
+            cap_drop=None,
+            privileged=None,
+            lxc_conf=None,
+            network_mode=None,
+            check_is_running=True,
+            publish_all_ports=False,
+            links=None,
+            restart_policy=None,
+            cpu_shares=None,
+            cpuset=None,
+            *args, **kwargs):
     '''
-    Ensure that a container is running. (`docker inspect`)
+    Ensure that a container is running. If the container does not exist, it
+    will be created from the specified image. (`docker run`)
 
-    name
-        name of the service
+    name / container
+        Name for the container
 
-    container
-        name of the container to start
+    image
+        Image from which to build this container
 
-    publish_all_ports
-
-    links
-        Link several container together
+    environment
+        Environment variables for the container, either
+            - a mapping of key, values
+            - a list of mappings of key, values
+    ports
+        List of ports definitions, either:
+            - a port to map
+            - a mapping of mapping portInHost : PortInContainer
 
         .. code-block:: yaml
 
-            - links:
-                name_other_container: alias_for_other_container
-
-    port_bindings
-        List of ports to expose on host system
-            - a mapping port's guest, hostname's host and port's host.
-
-        .. code-block:: yaml
-
-            - port_bindings:
-                "5000/tcp":
+            - ports:
+              - "5000/tcp":
                     HostIp: ""
                     HostPort: "5000"
 
-    binds
-        List of volumes to mount (like ``-v`` of ``docker run`` command),
-        mapping host directory to container directory.
+    publish_all_ports
+        Publish all ports from the port list (default is false,
+        only meaningful if port does not contain portinhost:portincontainer mapping)
 
-        For read-write mounting, use the short form:
+    volumes
+        List of volumes to mount or create in the container (like ``-v`` of ``docker run`` command),
+        mapping host directory to container directory.
+        To create a volume in the container:
 
         .. code-block:: yaml
 
-            - binds:
-                /var/log/service: /var/log/service
+            - volumes:
+              - "/var/log/service"
+
+        For read-write mounting, use the short form (note that the notion of
+        source:target from docker is preserved):
+
+        .. code-block:: yaml
+
+            - volumes:
+              - /var/log/service: /var/log/service
 
         Or, to specify read-only mounting, use the extended form:
 
         .. code-block:: yaml
 
-            - binds:
-                /home/user1:
-                    bind: /mnt/vol2
-                    ro: true
-                /var/www:
-                    bind: /mnt/vol1
-                    ro: false
+            - volumes:
+              - /home/user1:
+                bind: /mnt/vol2
+                ro: true
+              - /var/www:
+                bind: /mnt/vol1
+                ro: false
+
+        Or (mostly for backwards compatibility) a dict style
+
+        .. code-block:: yaml
+
+            - volumes:
+              /home/user1:
+                bind: /mnt/vol2
+                ro: true
+              /var/www:
+                bind: /mnt/vol1
+                ro: false
+
+    volumes_from
+        List of containers to share volumes with
 
     dns
         List of DNS servers.
@@ -702,14 +849,6 @@ def running(name, container=None, port_bindings=None, binds=None,
             - dns:
                 - 127.0.0.1
 
-    volumes_from
-        List of container names to get volumes definition from
-
-        .. code-block:: yaml
-
-            - volumes_from:
-                - name_other_container
-
     network_mode
         - 'bridge': creates a new network stack for the container on the docker bridge
         - 'none': no networking for this container
@@ -739,40 +878,156 @@ def running(name, container=None, port_bindings=None, binds=None,
         Enable checking if a container should run or not.
         Useful for data-only containers that must be linked to another one.
         e.g. nginx <- static-files
+
+    cpu_shares
+        CPU shares (relative weight)
+
+        .. code-block:: yaml
+
+            - cpu_shares: 2
+    cpuset
+        CPUs in which to allow execution ('0-3' or '0,1')
+
+        .. code-block:: yaml
+
+            - cpuset: '0-3'
+
+    For other parameters, see salt.modules.dockerio execution module
+    and the docker-py python bindings for docker documentation
+    <https://github.com/dotcloud/docker-py#api>`_ for
+    `docker.create_container`.
+
+    .. note::
+        This command does not verify that the named container
+        is running the specified image.
     '''
-    if not container and name:
-        container = name
-    is_running = __salt__['docker.is_running'](container)
-    if is_running:
-        return _valid(
-            comment='Container {0!r} is started'.format(container))
-    else:
-        started = __salt__['docker.start'](
-            container, binds=binds, port_bindings=port_bindings,
-            lxc_conf=lxc_conf, publish_all_ports=publish_all_ports,
-            links=links, privileged=privileged,
-            dns=dns, volumes_from=volumes_from, network_mode=network_mode,
-            restart_policy=restart_policy, cap_add=cap_add, cap_drop=cap_drop
-        )
+    if container is not None:
+        name = container
+    ins_image = __salt__['docker.inspect_image']
+    ins_container = __salt__['docker.inspect_container']
+    create = __salt__['docker.create_container']
+    iinfos = ins_image(image)
+    cinfos = ins_container(name)
+    already_exists = cinfos['status']
+    image_exists = iinfos['status']
+    is_running = False
+    if already_exists:
+        is_running = __salt__['docker.is_running'](container)
+    # if container exists but is not started, try to start it
+    if already_exists and (is_running or not start):
+        return _valid(comment='container {0!r} already exists'.format(name))
+    if not image_exists:
+        return _invalid(comment='image "{0}" does not exist'.format(image))
+    # parse input data
+    exposeports, bindports, contvolumes, bindvolumes, denvironment, changes = [], {}, [], {}, {}, []
+    if not ports:
+        ports = {}
+    if not volumes:
+        volumes = {}
+    if not volumes_from:
+        volumes_from = []
+    if isinstance(environment, dict):
+        for key in environment:
+            denvironment[six.text_type(key)] = six.text_type(environment[key])
+    if isinstance(environment, list):
+        for var in environment:
+            if isinstance(var, dict):
+                for key in var:
+                    denvironment[six.text_type(key)] = six.text_type(var[key])
+    if isinstance(volumes, dict):
+        bindvolumes = volumes
+    if isinstance(volumes, list):
+        for vol in volumes:
+            if isinstance(vol, dict):
+                # get source as the dict key
+                source = list(vol.keys())[0]
+                # then find target
+                if isinstance(vol[source], dict):
+                    target = vol[source]['bind']
+                    read_only = vol[source].get('ro', False)
+                else:
+                    target = str(vol[source])
+                    read_only = False
+                bindvolumes[source] = {
+                'bind': target,
+                'ro': read_only
+                }
+            else:
+                # assume just an own volumes
+                contvolumes.append(str(vol))
+    if isinstance(ports, dict):
+        bindports = ports
+        # in dict form all ports bind, so no need for exposeports
+    if isinstance(ports, list):
+        for port in ports:
+            if isinstance(port, dict):
+                container_port = list(port.keys())[0]
+                #find target
+                if isinstance(port[container_port], dict):
+                    host_port = port[container_port]['HostPort']
+                    host_ip = port[container_port].get('HostIp', '0.0.0.0')
+                else:
+                    host_port = str(port[container_port])
+                    host_ip = '0.0.0.0'
+                bindports[container_port] = {
+                    'HostPort': host_port,
+                    'HostIp': host_ip
+                }
+            else:
+                #assume just a port to expose
+                exposeports.append(str(port))
+    if not already_exists:
+        args, kwargs = [image], dict(
+                        command=command,
+                        hostname=hostname,
+                        user=user,
+                        detach=detach,
+                        stdin_open=stdin_open,
+                        tty=tty,
+                        mem_limit=mem_limit,
+                        ports=exposeports,
+                        environment=denvironment,
+                        dns=dns,
+                        volumes=contvolumes,
+                        name=name,
+                        cpu_shares=cpu_shares,
+                        cpuset=cpuset)
+        out = create(*args, **kwargs)
+        # if container has been created, even if not started, we mark
+        # it as installed
+        try:
+            cid = out['out']['info']['id']
+            log.debug(str(cid))
+        except Exception as e:
+            changes.append('Container created')
+            log.debug(str(e))
+        else:
+            changes.append('Container {0} created'.format(cid))
+    if start:
+        started = __salt__['docker.start'](name,
+                                           binds=bindvolumes,
+                                           port_bindings=bindports,
+                                           lxc_conf=lxc_conf,
+                                           publish_all_ports=publish_all_ports,
+                                           links=links,
+                                           privileged=privileged,
+                                           dns=dns,
+                                           volumes_from=volumes_from,
+                                           network_mode=network_mode,
+                                           restart_policy=restart_policy,
+                                           cap_add=cap_add,
+                                           cap_drop=cap_drop)
         if check_is_running:
-            is_running = __salt__['docker.is_running'](container)
+            is_running = __salt__['docker.is_running'](name)
             log.debug("Docker-io running:" + str(started))
             log.debug("Docker-io running:" + str(is_running))
             if is_running:
-                return _valid(
-                    comment='Container {0!r} started.\n'.format(container),
-                    changes={name: True})
+                changes.append('Container {0!r} started.\n'.format(name))
             else:
                 return _invalid(
-                    comment=(
+                        comment=(
                         'Container {0!r} cannot be started\n{1!s}'
-                        .format(
-                            container,
-                            started['out'],
-                        )
-                    )
-                )
+                        .format(name, started['out'],)))
         else:
-            return _valid(
-                comment='Container {0!r} started.\n'.format(container),
-                changes={name: True})
+            changes.append('Container {0!r} started.\n'.format(name))
+    return _valid(comment=','.join(changes), changes={name: True})
