@@ -14,6 +14,7 @@ authentication, it is also possible to pass private keys to use explicitly.
         - rev: develop
         - target: /tmp/salt
 '''
+from __future__ import absolute_import
 
 # Import python libs
 import logging
@@ -22,7 +23,6 @@ import os.path
 import shutil
 
 # Import salt libs
-import salt.utils
 from salt.exceptions import CommandExecutionError
 
 log = logging.getLogger(__name__)
@@ -38,7 +38,6 @@ def __virtual__():
 def latest(name,
            rev=None,
            target=None,
-           runas=None,
            user=None,
            force=None,
            force_checkout=False,
@@ -48,7 +47,10 @@ def latest(name,
            bare=False,
            remote_name='origin',
            always_fetch=False,
+           depth=None,
            identity=None,
+           https_user=None,
+           https_pass=None,
            onlyif=False,
            unless=False):
     '''
@@ -64,11 +66,6 @@ def latest(name,
     target
         Name of the target directory where repository is about to be cloned
 
-    runas
-        Name of the user performing repository management operations
-
-        .. deprecated:: 0.17.0
-
     user
         Name of the user performing repository management operations
 
@@ -81,6 +78,10 @@ def latest(name,
         Force a checkout even if there might be overwritten changes
         (Default: False)
 
+    force_reset
+        Force the checkout to ``--reset hard`` to the remote ref
+        (Default: False)
+
     submodules
         Update submodules on clone or branch change (Default: False)
 
@@ -102,8 +103,19 @@ def latest(name,
         until the tag or branch name changes. Setting this to true will force
         a fetch to occur. Only applies when rev is set. (Default: False)
 
+    depth
+        Defines depth in history when git a clone is needed in order to ensure
+        latest. E.g. ``depth: 1`` is usefull when deploying from a repository
+        with a long history. Use rev to specify branch. This is not compatible with tags or revision IDs.(Default: ``None``)
+
     identity
-        A path to a private key to use over SSH
+        A path on the minion server to a private key to use over SSH
+
+    https_user
+        HTTP Basic Auth username for HTTPS (only) clones
+
+    https_pass
+        HTTP Basic Auth password for HTTPS (only) clones
 
     onlyif
         A command to run as a check, run the named command only if the command
@@ -160,30 +172,7 @@ def latest(name,
 
     if not target:
         return _fail(ret, '"target" option is required')
-
-    salt.utils.warn_until(
-        'Lithium',
-        'Please remove \'runas\' support at this stage. \'user\' support was '
-        'added in 0.17.0',
-        _dont_call_warnings=True
-    )
-    if runas:
-        # Warn users about the deprecation
-        ret.setdefault('warnings', []).append(
-            'The \'runas\' argument is being deprecated in favor of \'user\', '
-            'please update your state files.'
-        )
-    if user is not None and runas is not None:
-        # user wins over runas but let warn about the deprecation.
-        ret.setdefault('warnings', []).append(
-            'Passed both the \'runas\' and \'user\' arguments. Please don\'t. '
-            '\'runas\' is being ignored in favor of \'user\'.'
-        )
-        runas = None
-    elif runas is not None:
-        # Support old runas usage
-        user = runas
-        runas = None
+    target = os.path.expanduser(target)
 
     run_check_cmd_kwargs = {'runas': user}
     if 'shell' in __grains__:
@@ -222,12 +211,15 @@ def latest(name,
             if (branch != 'HEAD' and branch == rev) or rev is None:
                 remote_rev = __salt__['git.ls_remote'](target,
                                                        repository=name,
-                                                       branch=branch, user=user,
-                                                       identity=identity)
+                                                       branch=branch,
+                                                       user=user,
+                                                       identity=identity,
+                                                       https_user=https_user,
+                                                       https_pass=https_pass)
 
             # only do something, if the specified rev differs from the
             # current_rev and remote_rev
-            if current_rev in [rev, remote_rev]:
+            if current_rev in [rev, remote_rev] or (remote_rev is not None and remote_rev.startswith(current_rev)):
                 new_rev = current_rev
             else:
 
@@ -252,8 +244,15 @@ def latest(name,
                     __salt__['git.remote_set'](target,
                                                name=remote_name,
                                                url=name,
-                                               user=user)
-                    ret['changes']['remote/{0}'.format(remote_name)] = "{0} => {1}".format(str(remote), name)
+                                               user=user,
+                                               https_user=https_user,
+                                               https_pass=https_pass)
+                    ret['changes']['remote/{0}'.format(remote_name)] = (
+                        "{0} => {1}".format(str(remote), name)
+                    )
+                    # Set to fetch later since we just added the remote and
+                    # need to get the refs
+                    always_fetch = True
 
                 # check if rev is already present in repo, git-fetch otherwise
                 if bare:
@@ -275,8 +274,9 @@ def latest(name,
                                               identity=identity)
 
                     if force_reset:
+                        opts = "--hard {0}/{1}".format(remote_name, rev)
                         __salt__['git.reset'](target,
-                                              opts="--hard",
+                                              opts=opts,
                                               user=user)
 
                     __salt__['git.checkout'](target,
@@ -285,30 +285,42 @@ def latest(name,
                                              user=user)
 
                     if branch != 'HEAD':
-                        current_remote = __salt__['git.config_get'](target,
-                                                             'branch.{0}.remote'.format(rev),
-                                                             user=user)
+                        try:
+                            current_remote = __salt__['git.config_get'](
+                                target, 'branch.{0}.remote'.format(rev),
+                                user=user)
+                        except CommandExecutionError:
+                            current_remote = None
+
                         if current_remote != remote_name:
                             if __opts__['test']:
-                                ret['changes'] = {'old': current_remote, 'new': remote_name}
-                                return _neutral_test(ret,
-                                                     ('Repository {0} update is probably required.'
-                                                      'Current remote is {1} should be {2}'.format(target, current_remote, remote_name)))
-                            log.debug('Setting branch {0} to upstream {1}'.format(rev, remote_name))
-                            __salt__['git.branch'](target,
-                                                   rev,
-                                                   opts='--set-upstream {0}/{1}'.format(remote_name, rev),
-                                                   user=user)
-                            ret['changes']['remote/{0}/{1}'.format(remote_name, rev)] = '{0} => {1}'.format(current_remote, remote_name)
+                                ret['changes'] = {'old': current_remote,
+                                                  'new': remote_name}
+                                return _neutral_test(
+                                    ret,
+                                    ('Repository {0} update is probably '
+                                     'required. Current remote is {1} should '
+                                     'be {2}'.format(target, current_remote,
+                                                     remote_name)))
+                            log.debug(
+                                'Setting branch {0} to '
+                                'upstream {1}'.format(rev, remote_name))
+                            __salt__['git.branch'](
+                                target, rev,
+                                opts='--set-upstream '
+                                '{0}/{1}'.format(remote_name, rev),
+                                user=user)
+                            ret['changes']['remote/''{0}/{1}'.format(
+                                remote_name, rev)] = (
+                                    '{0} => {1}'.format(current_remote,
+                                                        remote_name))
 
                 # check if we are on a branch to merge changes
                 cmd = "git symbolic-ref -q HEAD"
                 retcode = __salt__['cmd.retcode'](cmd, cwd=target, runas=user)
                 if 0 == retcode:
-                    __salt__['git.fetch' if bare else 'git.pull'](target,
-                                                                  opts=fetch_opts,
-                                                                  user=user,
-                                                                  identity=identity)
+                    __salt__['git.fetch' if bare else 'git.pull'](
+                        target, opts=fetch_opts, user=user, identity=identity)
 
                 if submodules:
                     __salt__['git.submodule'](target,
@@ -321,9 +333,10 @@ def latest(name,
                 except CommandExecutionError:
                     new_rev = None
         except Exception as exc:
+            log.error('Unexpected exception in git state', exc_info=True)
             return _fail(
-                    ret,
-                    str(exc))
+                ret,
+                str(exc))
 
         if current_rev != new_rev:
             log.info('Repository {0} updated: {1} => {2}'.format(target,
@@ -331,7 +344,7 @@ def latest(name,
                                                                  new_rev))
             ret['comment'] = 'Repository {0} updated'.format(target)
             ret['changes']['revision'] = '{0} => {1}'.format(
-                    current_rev, new_rev)
+                current_rev, new_rev)
     else:
         if os.path.isdir(target):
             # git clone is required, target exists but force is turned on
@@ -344,31 +357,40 @@ def latest(name,
                     shutil.rmtree(target)
             # git clone is required, but target exists and is non-empty
             elif os.listdir(target):
-                return _fail(ret, 'Directory \'{0}\' exists, is non-empty, and '
-                             'force option not in use'.format(target))
+                return _fail(ret, 'Directory \'{0}\' exists, is non-empty, '
+                             'and force option not in use'.format(target))
 
         # git clone is required
         log.debug(
-                'target {0} is not found, "git clone" is required'.format(
-                    target))
+            'target {0} is not found, "git clone" is required'.format(
+                target))
         if 'test' in __opts__:
             if __opts__['test']:
                 return _neutral_test(
-                        ret,
-                        'Repository {0} is about to be cloned to {1}'.format(
-                            name, target))
+                    ret,
+                    'Repository {0} is about to be cloned to {1}'.format(
+                        name, target))
         try:
             # make the clone
             opts = '--mirror' if mirror else '--bare' if bare else ''
             # if remote_name is not origin add --origin <name> to opts
             if remote_name != 'origin':
                 opts += ' --origin {0}'.format(remote_name)
+
+            # if depth is given add --depth <depth> to opts
+            if depth is not None:
+                opts += ' --depth {0}'.format(depth)
+                if rev is not None:
+                    opts += ' --branch {0}'.format(rev)
+
             # do the clone
             __salt__['git.clone'](target,
                                   name,
                                   user=user,
                                   opts=opts,
-                                  identity=identity)
+                                  identity=identity,
+                                  https_user=https_user,
+                                  https_pass=https_pass)
 
             if rev and not bare:
                 __salt__['git.checkout'](target, rev, user=user)
@@ -387,9 +409,10 @@ def latest(name,
                     pass
 
         except Exception as exc:
+            log.error('Unexpected exception in git state', exc_info=True)
             return _fail(
-                    ret,
-                    str(exc))
+                ret,
+                str(exc))
 
         message = 'Repository {0} cloned to {1}'.format(name, target)
         log.info(message)
@@ -400,7 +423,7 @@ def latest(name,
     return ret
 
 
-def present(name, bare=True, runas=None, user=None, force=False):
+def present(name, bare=True, user=None, force=False, shared=None):
     '''
     Make sure the repository is present in the given directory
 
@@ -410,11 +433,6 @@ def present(name, bare=True, runas=None, user=None, force=False):
     bare
         Create a bare repository (Default: True)
 
-    runas
-        Name of the user performing repository management operations
-
-        .. deprecated:: 0.17.0
-
     user
         Name of the user performing repository management operations
 
@@ -423,32 +441,14 @@ def present(name, bare=True, runas=None, user=None, force=False):
     force
         Force-create a new repository into an pre-existing non-git directory
         (deletes contents)
-    '''
-    ret = {'name': name, 'result': True, 'comment': '', 'changes': {}}
 
-    salt.utils.warn_until(
-        'Lithium',
-        'Please remove \'runas\' support at this stage. \'user\' support was '
-        'added in 0.17.0',
-        _dont_call_warnings=True
-    )
-    if runas:
-        # Warn users about the deprecation
-        ret.setdefault('warnings', []).append(
-            'The \'runas\' argument is being deprecated in favor of \'user\', '
-            'please update your state files.'
-        )
-    if user is not None and runas is not None:
-        # user wins over runas but let warn about the deprecation.
-        ret.setdefault('warnings', []).append(
-            'Passed both the \'runas\' and \'user\' arguments. Please don\'t. '
-            '\'runas\' is being ignored in favor of \'user\'.'
-        )
-        runas = None
-    elif runas is not None:
-        # Support old runas usage
-        user = runas
-        runas = None
+    shared
+        Specify the permission for sharing, see git-init for details (Default: None)
+
+       .. versionadded:: XXXX
+    '''
+    name = os.path.expanduser(name)
+    ret = {'name': name, 'result': True, 'comment': '', 'changes': {}}
 
     # If the named directory is a git repo return True
     if os.path.isdir(name):
@@ -478,6 +478,7 @@ def present(name, bare=True, runas=None, user=None, force=False):
             shutil.rmtree(name)
 
     opts = '--bare' if bare else ''
+    opts += ' --shared={0}'.format(shared) if shared else ''
     __salt__['git.init'](cwd=name, user=user, opts=opts)
 
     message = 'Initialized repository {0}'.format(name)
