@@ -3,6 +3,7 @@
 # Import Python Libs
 import json
 import time
+from distutils.version import StrictVersion  # pylint: disable=import-error,no-name-in-module
 
 # Import Salt Libs
 from salt.netapi.rest_tornado import saltnado
@@ -21,6 +22,7 @@ except ImportError:
     HAS_TORNADO = False
 
 try:
+    import zmq
     from zmq.eventloop.ioloop import ZMQIOLoop
     HAS_ZMQ_IOLOOP = True
 except ImportError:
@@ -29,6 +31,7 @@ except ImportError:
 
 @skipIf(HAS_TORNADO is False, 'Tornado must be installed to run these tests')
 @skipIf(HAS_ZMQ_IOLOOP is False, 'PyZMQ version must be >= 14.0.1 to run these tests.')
+@skipIf(StrictVersion(zmq.__version__) < StrictVersion('14.0.1'), 'PyZMQ must be >= 14.0.1 to run these tests.')
 class TestSaltAPIHandler(SaltnadoTestCase):
     def get_app(self):
         application = tornado.web.Application([('/', saltnado.SaltAPIHandler)], debug=True)
@@ -190,11 +193,15 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                               connect_timeout=10,
                               request_timeout=10,
                               )
+
         response_obj = json.loads(response.body)
+        ret = response_obj['return']
+        ret[0]['minions'] = sorted(ret[0]['minions'])
+
         # TODO: verify pub function? Maybe look at how we test the publisher
-        self.assertEqual(len(response_obj['return']), 1)
-        self.assertIn('jid', response_obj['return'][0])
-        self.assertEqual(set(response_obj['return'][0]['minions']), set(['minion', 'sub_minion']))
+        self.assertEqual(len(ret), 1)
+        self.assertIn('jid', ret[0])
+        self.assertEqual(ret[0]['minions'], sorted(['minion', 'sub_minion']))
 
     def test_multi_local_async_post(self):
         low = [{'client': 'local_async',
@@ -213,12 +220,17 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                               connect_timeout=10,
                               request_timeout=10,
                               )
+
         response_obj = json.loads(response.body)
-        self.assertEqual(len(response_obj['return']), 2)
-        self.assertIn('jid', response_obj['return'][0])
-        self.assertIn('jid', response_obj['return'][1])
-        self.assertEqual(set(response_obj['return'][0]['minions']), set(['minion', 'sub_minion']))
-        self.assertEqual(set(response_obj['return'][1]['minions']), set(['minion', 'sub_minion']))
+        ret = response_obj['return']
+        ret[0]['minions'] = sorted(ret[0]['minions'])
+        ret[1]['minions'] = sorted(ret[1]['minions'])
+
+        self.assertEqual(len(ret), 2)
+        self.assertIn('jid', ret[0])
+        self.assertIn('jid', ret[1])
+        self.assertEqual(ret[0]['minions'], sorted(['minion', 'sub_minion']))
+        self.assertEqual(ret[1]['minions'], sorted(['minion', 'sub_minion']))
 
     def test_multi_local_async_post_multitoken(self):
         low = [{'client': 'local_async',
@@ -244,13 +256,18 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                               connect_timeout=10,
                               request_timeout=10,
                               )
+
         response_obj = json.loads(response.body)
-        self.assertEqual(len(response_obj['return']), 3)  # make sure we got 3 responses
-        self.assertIn('jid', response_obj['return'][0])  # the first 2 are regular returns
-        self.assertIn('jid', response_obj['return'][1])
-        self.assertIn('Failed to authenticate', response_obj['return'][2])  # bad auth
-        self.assertEqual(set(response_obj['return'][0]['minions']), set(['minion', 'sub_minion']))
-        self.assertEqual(set(response_obj['return'][1]['minions']), set(['minion', 'sub_minion']))
+        ret = response_obj['return']
+        ret[0]['minions'] = sorted(ret[0]['minions'])
+        ret[1]['minions'] = sorted(ret[1]['minions'])
+
+        self.assertEqual(len(ret), 3)  # make sure we got 3 responses
+        self.assertIn('jid', ret[0])  # the first 2 are regular returns
+        self.assertIn('jid', ret[1])
+        self.assertIn('Failed to authenticate', ret[2])  # bad auth
+        self.assertEqual(ret[0]['minions'], sorted(['minion', 'sub_minion']))
+        self.assertEqual(ret[1]['minions'], sorted(['minion', 'sub_minion']))
 
     def test_simple_local_async_post_no_tgt(self):
         low = [{'client': 'local_async',
@@ -340,11 +357,15 @@ class TestMinionSaltAPIHandler(SaltnadoTestCase):
                               connect_timeout=10,
                               request_timeout=10,
                               )
+
         response_obj = json.loads(response.body)
+        ret = response_obj['return']
+        ret[0]['minions'] = sorted(ret[0]['minions'])
+
         # TODO: verify pub function? Maybe look at how we test the publisher
-        self.assertEqual(len(response_obj['return']), 1)
-        self.assertIn('jid', response_obj['return'][0])
-        self.assertEqual(set(response_obj['return'][0]['minions']), set(['minion', 'sub_minion']))
+        self.assertEqual(len(ret), 1)
+        self.assertIn('jid', ret[0])
+        self.assertEqual(ret[0]['minions'], sorted(['minion', 'sub_minion']))
 
     def test_post_with_client(self):
         # get a token for this test
@@ -360,11 +381,15 @@ class TestMinionSaltAPIHandler(SaltnadoTestCase):
                               connect_timeout=10,
                               request_timeout=10,
                               )
+
         response_obj = json.loads(response.body)
+        ret = response_obj['return']
+        ret[0]['minions'] = sorted(ret[0]['minions'])
+
         # TODO: verify pub function? Maybe look at how we test the publisher
-        self.assertEqual(len(response_obj['return']), 1)
-        self.assertIn('jid', response_obj['return'][0])
-        self.assertEqual(set(response_obj['return'][0]['minions']), set(['minion', 'sub_minion']))
+        self.assertEqual(len(ret), 1)
+        self.assertIn('jid', ret[0])
+        self.assertEqual(ret[0]['minions'], sorted(['minion', 'sub_minion']))
 
     def test_post_with_incorrect_client(self):
         '''
@@ -412,13 +437,17 @@ class TestJobsSaltAPIHandler(SaltnadoTestCase):
                                )
         response = self.wait(timeout=10)
         response_obj = json.loads(response.body)['return'][0]
-        for jid, ret in response_obj.iteritems():
-            self.assertIn('Function', ret)
-            self.assertIn('Target', ret)
-            self.assertIn('Target-type', ret)
-            self.assertIn('User', ret)
-            self.assertIn('StartTime', ret)
-            self.assertIn('Arguments', ret)
+        try:
+            for jid, ret in response_obj.iteritems():
+                self.assertIn('Function', ret)
+                self.assertIn('Target', ret)
+                self.assertIn('Target-type', ret)
+                self.assertIn('User', ret)
+                self.assertIn('StartTime', ret)
+                self.assertIn('Arguments', ret)
+        except AttributeError as attribute_error:
+            print json.loads(response.body)
+            raise
 
         # test with a specific JID passed in
         jid = response_obj.iterkeys().next()
@@ -560,3 +589,13 @@ class TestWebhookSaltAPIHandler(SaltnadoTestCase):
                               )
         response_obj = json.loads(response.body)
         self.assertTrue(response_obj['success'])
+
+
+if __name__ == '__main__':
+    from integration import run_tests  # pylint: disable=import-error
+    run_tests(TestEventsSaltAPIHandler,
+              TestJobsSaltAPIHandler,
+              TestMinionSaltAPIHandler,
+              TestRunSaltAPIHandler,
+              TestSaltAPIHandler,
+              TestWebhookSaltAPIHandler, needs_daemon=True)
