@@ -159,7 +159,7 @@ class ConfigTestCase(TestCase, integration.AdaptedConfigurationTestCaseMixIn):
                 'log_file: {1}\n'.format(root_dir, fpath)
             )
             # Let's set the environment variable, yet, since the configuration
-            # file path is not the default one, ie, the user has passed an
+            # file path is not the default one, i.e., the user has passed an
             # alternative configuration file form the CLI parser, the
             # environment variable will be ignored.
             os.environ['SALT_MASTER_CONFIG'] = env_fpath
@@ -201,7 +201,7 @@ class ConfigTestCase(TestCase, integration.AdaptedConfigurationTestCaseMixIn):
                 'log_file: {1}\n'.format(root_dir, fpath)
             )
             # Let's set the environment variable, yet, since the configuration
-            # file path is not the default one, ie, the user has passed an
+            # file path is not the default one, i.e., the user has passed an
             # alternative configuration file form the CLI parser, the
             # environment variable will be ignored.
             os.environ['SALT_MINION_CONFIG'] = env_fpath
@@ -255,7 +255,7 @@ class ConfigTestCase(TestCase, integration.AdaptedConfigurationTestCaseMixIn):
                 'log_file: {1}\n'.format(root_dir, fpath)
             )
             # Let's set the environment variable, yet, since the configuration
-            # file path is not the default one, ie, the user has passed an
+            # file path is not the default one, i.e., the user has passed an
             # alternative configuration file form the CLI parser, the
             # environment variable will be ignored.
             os.environ['SALT_MASTER_CONFIG'] = env_fpath
@@ -356,8 +356,8 @@ class ConfigTestCase(TestCase, integration.AdaptedConfigurationTestCaseMixIn):
         self.assertEqual(syndic_opts['master'], 'localhost')
         self.assertEqual(syndic_opts['sock_dir'], os.path.join(root_dir, 'minion_sock'))
         self.assertEqual(syndic_opts['cachedir'], os.path.join(root_dir, 'cache'))
-        self.assertEqual(syndic_opts['log_file'], os.path.join(root_dir, 'var/log/salt/syndic'))
-        self.assertEqual(syndic_opts['pidfile'], os.path.join(root_dir, 'var/run/salt-syndic.pid'))
+        self.assertEqual(syndic_opts['log_file'], os.path.join(root_dir, 'osyndic.log'))
+        self.assertEqual(syndic_opts['pidfile'], os.path.join(root_dir, 'osyndic.pid'))
         # Show that the options of localclient that repub to local master
         # are not merged with syndic ones
         self.assertEqual(syndic_opts['_master_conf_file'], minion_conf_path)
@@ -871,7 +871,7 @@ class ConfigTestCase(TestCase, integration.AdaptedConfigurationTestCaseMixIn):
                 'log_file: {1}\n'.format(root_dir, fpath)
             )
             # Let's set the environment variable, yet, since the configuration
-            # file path is not the default one, ie, the user has passed an
+            # file path is not the default one, i.e., the user has passed an
             # alternative configuration file form the CLI parser, the
             # environment variable will be ignored.
             os.environ['SALT_CLOUD_CONFIG'] = env_fpath
