@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 
 # Import python libs
-import warnings
 
 # Import Salt Testing libs
 from salttesting import skipIf, TestCase
@@ -29,6 +28,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -40,7 +40,7 @@ class PipTestCase(TestCase):
                 pip.install,
                 editable='git+https://github.com/saltstack/salt-testing.git'
             )
-            #mock.assert_called_once_with('', runas=None, cwd=None)
+            #mock.assert_called_once_with('', cwd=None, use_vt=False)
 
     def test_install_multiple_editable(self):
         editables = [
@@ -59,6 +59,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -73,6 +74,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -98,6 +100,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -112,6 +115,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -125,6 +129,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -147,6 +152,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -162,6 +168,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -175,6 +182,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -197,6 +205,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -212,6 +221,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -224,6 +234,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -255,6 +266,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -265,7 +277,7 @@ class PipTestCase(TestCase):
                 CommandExecutionError,
                 pip.install, no_index=True, index_url='http://foo.tld'
             )
-            #mock.assert_called_once_with('', runas=None, cwd=None)
+            #mock.assert_called_once_with('', cwd=None)
 
         mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
         with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
@@ -273,7 +285,7 @@ class PipTestCase(TestCase):
                 CommandExecutionError,
                 pip.install, no_index=True, extra_index_url='http://foo.tld'
             )
-            #mock.assert_called_once_with('', runas=None, cwd=None)
+            #mock.assert_called_once_with('', cwd=None)
 
     @patch('salt.modules.pip._get_cached_requirements')
     def test_install_failed_cached_requirements(self, get_cached_requirements):
@@ -294,6 +306,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -315,8 +328,9 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
-                )
+            )
 
     @patch('os.path')
     def test_install_log_argument_in_resulting_command(self, mock_path):
@@ -328,6 +342,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -352,6 +367,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -364,6 +380,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -386,6 +403,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -398,6 +416,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -410,6 +429,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -422,6 +442,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -434,6 +455,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -446,6 +468,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -458,6 +481,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -470,6 +494,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -482,6 +507,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -495,6 +521,7 @@ class PipTestCase(TestCase):
                     saltenv='base',
                     runas=None,
                     cwd=None,
+                    use_vt=False,
                     python_shell=False,
                 )
 
@@ -525,6 +552,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -539,6 +567,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -552,6 +581,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -572,6 +602,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -586,6 +617,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -598,6 +630,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -610,6 +643,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -622,6 +656,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -634,6 +669,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -646,6 +682,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -658,6 +695,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -671,6 +709,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -694,6 +733,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -711,6 +751,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -724,6 +765,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -747,6 +789,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -764,6 +807,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -777,6 +821,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -792,6 +837,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -805,6 +851,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -829,6 +876,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -841,6 +889,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -874,6 +923,7 @@ class PipTestCase(TestCase):
                 'pip freeze',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
             self.assertEqual(ret, eggs)
@@ -972,6 +1022,7 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
@@ -988,113 +1039,10 @@ class PipTestCase(TestCase):
                 saltenv='base',
                 runas=None,
                 cwd=None,
+                use_vt=False,
                 python_shell=False,
             )
 
-    def test_install_deprecated_runas_triggers_warning(self):
-        # We *always* want *all* warnings thrown on this module
-        warnings.resetwarnings()
-        warnings.filterwarnings('always', '', DeprecationWarning, __name__)
-
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            with warnings.catch_warnings(record=True) as w:
-                pip.install('pep8', runas='me!')
-                self.assertEqual(
-                    'The \'runas\' argument to pip.install is deprecated, and '
-                    'will be removed in Salt Lithium (Unreleased). Please '
-                    'use \'user\' instead.', str(w[-1].message)
-                )
-
-    def test_uninstall_deprecated_runas_triggers_warning(self):
-        # We *always* want *all* warnings thrown on this module
-        warnings.resetwarnings()
-        warnings.filterwarnings('always', '', DeprecationWarning, __name__)
-
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            with warnings.catch_warnings(record=True) as w:
-                pip.uninstall('pep8', runas='me!')
-                self.assertEqual(
-                    'The \'runas\' argument to pip.install is deprecated, and '
-                    'will be removed in Salt Lithium (Unreleased). Please '
-                    'use \'user\' instead.', str(w[-1].message)
-                )
-
-    def test_freeze_deprecated_runas_triggers_warning(self):
-        # We *always* want *all* warnings thrown on this module
-        warnings.resetwarnings()
-        warnings.filterwarnings('always', '', DeprecationWarning, __name__)
-
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            with warnings.catch_warnings(record=True) as w:
-                pip.freeze('/tmp/pip-env', runas='me!')
-                self.assertEqual(
-                    'The \'runas\' argument to pip.install is deprecated, and '
-                    'will be removed in Salt Lithium (Unreleased). Please '
-                    'use \'user\' instead.', str(w[-1].message)
-                )
-
-    def test_list_deprecated_runas_triggers_warning(self):
-        # We *always* want *all* warnings thrown on this module
-        warnings.resetwarnings()
-        warnings.filterwarnings('always', '', DeprecationWarning, __name__)
-
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': 'pip VERSION'})
-        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            with warnings.catch_warnings(record=True) as w:
-                pip.list_('blah', runas='me!')
-                self.assertEqual(
-                    'The \'runas\' argument to pip.install is deprecated, and '
-                    'will be removed in Salt Lithium (Unreleased). Please '
-                    'use \'user\' instead.', str(w[-1].message)
-                )
-
-    def test_install_user_and_runas_raises_exception(self):
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            self.assertRaises(
-                CommandExecutionError,
-                pip.install,
-                'pep8',
-                user='Me!',
-                runas='Not Me!',
-            )
-
-    def test_uninstall_user_and_runas_raises_exception(self):
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            self.assertRaises(
-                CommandExecutionError,
-                pip.uninstall,
-                'pep8',
-                user='Me!',
-                runas='Not Me!',
-            )
-
-    def test_freeze_user_and_runas_raises_exception(self):
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            self.assertRaises(
-                CommandExecutionError,
-                pip.freeze,
-                '/tmp/pip-env',
-                user='Me!',
-                runas='Not Me!',
-            )
-
-    def test_list_user_and_runas_raises_exception(self):
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            self.assertRaises(
-                CommandExecutionError,
-                pip.list_,
-                'pep8',
-                user='Me!',
-                runas='Not Me!',
-            )
-
 
 if __name__ == '__main__':
     from integration import run_tests
